function burger
% Solve the nonlinear Schroedinger equation using using
% the FFT split-step method
%
%  Ut  + mu*Ux*U = nu*Uxx
%
%  U(0, t) = U(1, t)
%  U(x, 0) = U_0
%
%
%  OMN - 1997
%

%------------------
% Method parameters
%------------------

epsilon = 1.0e-15;
slowiter = 25;
maxiter = 50;

%------------------
% Model parameters
%------------------
n                = 10;                         % 4 if p = 5
Ld               = 1;                        % Domain size             
N                = Ld*2^n;                    % Grid resolution 

nu               = 0.002;                     % Diffusion parameter
mu               = 1;                         % Non-linear advection
%T                = 2^9;
T                = 200;
finaltime        = 0.5;                       %
dt               = finaltime/T;               % Timestep
time             = 0.0;                       % Simulated time 

plotinterval     = finaltime/8;              % Time between plots
stepsbetweenplot = round(plotinterval/dt);


%----------------------
% Abscissae: x in [-L/2:L/2]
%----------------------
h = Ld/N;   
x = (0:h:Ld-h)';  % N sample points 

do     = 1/Ld;                     % Frequency interval. 
Omega  = (-N/2/Ld:do:N/2/Ld-do)';  % Frequency-vector (Abscissae). 


D1 = i*2*pi*Omega;
D1 = fftshift(D1); 
L  = -nu*(2*pi*Omega).^2;    % Linear operator in Fourier domain
E  = exp(dt/2*L);            % Precompute exponential (half timestep)
E  = fftshift(E);

%%%%%%% Initial condition:

u = sin(2*pi*x/Ld);

uana1 = 2*pi/Ld*cos(2*pi*x/Ld); 

figure(1); plot(x,u)
U  = fft(u);
Ux = D1.*U;
ux = ifft(Ux);
figure(2); plot(x,real(ux),x,uana1)

%return



fprintf('N=%d, interval = %6.2f (%d steps), %s\n',N,plotinterval,stepsbetweenplot,'FFT');

%%%%%%% Predict solution for each time step.
%count = 1;
k     = 0;

% Nonlinear part 
U  = fft(u);     % Fourier transform of u.
Ux = D1.*U;      % Apply linear operator
ux = ifft(Ux);   % Transform back to the spatial domain.
N1 = -mu*real(ux);

flops_tot = 0;
f0        = flops;
cpu_tot   = 0;
c0        = cputime;
while time < finaltime,

  if k==0 | k==stepsbetweenplot
    figure(1); clf; plot(x,real(u));
    %figure(2); clf; plot(x,imag(u));    
    title('FFT-SPLIT STEP');
    xlabel('x');
    ylabel('u');    
    
    fprintf ('time = %6.3f, Norm(u) = %f\n',time,norm(u)*sqrt(h)); 
    k = 0;
    if mu == 0
      uana = exp(-nu*time*4*pi^2) * sin(2*pi*x);
      norm(u-uana,inf)
    end;  
  end

  %%% FFT half timestep
  u0 = u;                % Value at z
  u1 = u0;               % Value at z+dz (approx)
      
  U0    = fft(u0);       % Fourier transform of u.
  Uhalf = E.*U0;         % Apply linear operator
  uhalf = ifft(Uhalf);   % Transform back to the spatial domain.
  
  % Iterate
  iter  = 0;
  error = 1;  
  N0 = N1;     % Use nonlinear part from the end of previous step    
  while error > epsilon
    u1old = u1;
  
    % Nonlinear part at t + dt (approx)  
    U1  = fft(u1);     % Fourier transform of u.
    U1x = D1.*U1;      % Apply linear operator
    u1x = ifft(U1x);   % Transform back to the spatial domain.
    N1  = -mu*real(u1x);

    Q    = (N0 + N1) * dt/2;       % Trapez formula
    u_nl = exp(Q).*uhalf;          % Apply non-linearity
    
    U_nl = fft(u_nl);    % Fourier transform of u_nl.
    U1   = E.*U_nl;      % Apply linear operator
    u1   = ifft(U1);     % Transform back to the spatial domain

    error = norm(u1-u1old,inf)/norm(u1,inf);
    iter = iter + 1;
    if iter > slowiter
      fprintf('Warning - Slow convergence, iteration # = %d\n',iter);   
      if iter > maxiter
        fprintf('Split step does not converge\n');
        return
      end;
    end;         
  end;  
%  fprintf('Err = %e, Iterations=%d\n',error,iter);
  u = u1; 
 
  %------------------------------
  % Advance time and step counter
  %------------------------------
  time = time + dt;            
  k    = k+1;  

end %while  

flops_tot=(flops-f0)/1.0e6; 
cpu_tot = (cputime-c0);
fprintf('MFlops = %8.2f, time = %10d\n',flops_tot,cpu_tot);









