function burger
% Solve Burger's equation using WOFD
%
%  Ut + mu*U*Ux = nu*Uxx
%
%  U( 0, t) = U(2 pi, t)
%  U( x, 0) = sin(2 pi x)
%
%
% Usage:  burger;
% 
%
% OMN, May 1997

fprintf('Demonstration of the Wavelet Optimized Finite Difference method.\n');
fprintf('Burgers equation is solved, and the grid is adapted to the shape\n');
fprintf('of the solution using the WOFD method.\n');
fprintf('A reference solution is also computed and the error is shown.\n\n');
%fprintf('Hence the speedup achieved by reduction of points will not be seen')
%fprintf('Press a key to continue.\n');
%pause


figure(1); clf
figure(2); clf
fs = 16; %Fontsize

%------------------
% Method parameters
%------------------
p                = 3;         % FD stencil (must be odd)
D                = 6;         % Wavelet genus
epsilon          = 1.0e-6;    % Threshold for wavelet coefs
interpol         = '*spline'; % Interpolation, 'spline' or 'linear'
redothreshold    = 0.02;      % redo when sum(added)/length(added) > redothresh
adaptivetime     = 0;         % Use adaptive time (experimental)
saveplots        = 0;         % Make eps files of graphs

%------------------
% Model parameters
%------------------
n                = 10;                        % Resolution
N                = 2^n;                       % Finest grid 
L                = 1;                         % Domain size

%rho              = 0.2;                       % Linear advection
rho              = 0.0;                       % Linear advection

nu               = 0.002;                     % Diffusion parameter
mu               = 1;                         % Non-linear advection
T                = 2^9;
finaltime        = 0.5;                       %
dt               = finaltime/T;               % Timestep
time             = 0.0;                       % Simulated time 
plotinterval     = finaltime/8;               % Time between evaluations 
stepsbetweenplot = round(plotinterval/dt);


%----------------------
% Abscissae: x in [0,L]
%----------------------
h = L/N;   
x = (0:h:L-h)';  % N sample points 

%-------------------
% Initial condition:
%-------------------
u  = sin(2*pi*x);

%-----------------
% Setup reference:
%----------------- 
I         = speye(N);            % Sparse NxN identy matrix
[Dx,Dxx]  = fdmat(x,L,p);        % Generate FD operators
A         = I - 0.5*dt*nu*Dxx;   % Crank Nicolson matrices
E         = 2*I - A;             % -"-
[Low, Up] = lu(A);               % LU decomp of A 
uref      = u;                   % Reference solution

%-----------
% Initialize
%-----------
xc   = x;  %Initially, restricted grid is the same as the finest.
uc   = u;
Dxc  = Dx;
Dxxc = Dxx;
ind  = ones(N,1);
Nc   = length(xc);

fprintf('N=%d, interval = %6.4f (%d steps), D=%d, p=%d, %s\n',N,plotinterval,stepsbetweenplot,D,p,interpol);
%fprintf('                                         |-------- New Grid ----------|\n')
%fprintf(' time(interval)   error       #points    #points    #added     #removed\n')
fprintf('                                |-------- New Grid --------|\n')
fprintf(' time     error       #points   #points   #added    #removed\n')

redoing = 0;  % Reset flag for redoing steps
k       = 0;  % Reset Step counter

%-------------------------
% Compute until final time
%-------------------------

flops_eval      = 0;
flops_solve     = 0;
flops_solve_ref = 0;
cpu_eval        = 0;
cpu_solve       = 0;
cpu_solve_ref   = 0;

while time < finaltime 
  if k==0 | k==stepsbetweenplot  % Evaluate grid

    f0 = flops;
    c0 = cputime;
   
    %-------------------------------
    % Reconstruct u on finest grid x
    %-------------------------------    

    aug=min(8,Nc);
    xcP = [xc(Nc-aug+1:Nc)-L;xc;xc(1:aug)+L];
    ucP = [uc(Nc-aug+1:Nc);uc;uc(1:aug)];    
    u = interp1(xcP,ucP,x,interpol);

    if any(isnan(u))
       fprintf('WARNING\n');
    end

    err=max(abs(uref-u))/max(abs(u));    
    figure(2); plot(x,(uref-u)/max(abs(u))); title('Error','Fontsize',fs);
    set(gca,'Fontsize',[fs])
    if saveplots
      eval (['print -depsc2 burger' num2str(time) '_' num2str(rho) '_e.eps']);
    end;  

    
    %----------------
    % Define new grid
    %----------------
    oldind=ind;

    ind     = makegrid(u,D,epsilon); % Wavelet based grid evaluator
    changed = xor(ind,oldind);
    added   = changed & ind;

    %
    % Avoid stagnation
    % Don't remove any if we are redoing the step and still adding!
    %
    if any(added) & redoing  
      ind = added | oldind;
      changed = xor(ind,oldind);
    end
    removed = changed & oldind;


    %------------------------ 
    % Plot grid modifications
    %------------------------

    % Plot restricted uc 
    %points = zeros(size(xc));  % for plot     
    %figure(1); clf; subplot(3,1,1); plot(x,u,x,uref,xc,points,'o'); 
    %title('Old grid')
    
    figure(1);clf;subplot(2,1,2) 
    set(gca,'Fontsize',[fs])    
    hold on
    if any(added)
      xa = x(find(added));
      na = sum(added);
      plot(xa,ones(na,1)/2,'g+',xa,nan*ones(na,1),'r.');
      legend('Pts added', 'Pts removed')
    end;
    if any(removed)
      xr = x(find(removed));
      nr = sum(removed);
      plot(xr,nan*ones(nr,1),'g+',xr,ones(nr,1)/3,'r.');
      legend('Pts added', 'Pts removed')      
    end; 
    axis([0 L 0 1]);
    title('Grid modifications','Fontsize',fs)
    hold off

    xc = x(find(ind));
    Nc = length(xc);
    
    %  Quality
    %
    %
    addratio = sum(added)/N;
    remratio = sum(removed)/N; 
    if adaptivetime   
      plotinterval = plotinterval/(1+addratio)*(1+remratio);          % Time between plots
      stepsbetweenplot = round(plotinterval/dt);
    end  
    
%    fprintf('%5.2f (%5.2f) %12.8f %7d %9d %9d %10d ',time,plotinterval,err,sum(oldind),Nc,sum(added),sum(removed))             
    fprintf('%5.4f %12.8f %7d %9d %9d %10d ',time,err,sum(oldind),Nc,sum(added),sum(removed))             
    
    if addratio > redothreshold  % Redo steps on refined grid
      fprintf(' - Redoing step');
      redoing = 1;
      u    = uold;
      time = timeold;
      uref = urefold; 
    else
      fprintf(' ');
      redoing = 0;
      uold    = u;
      timeold = time;
      urefold = uref;
    end 

    %----------------------------
    % Restrict u to new grid (xc)
    %----------------------------
    uc = u(find(ind));
    points = zeros(size(xc));  % for plots

    figure(1); subplot(2,1,1); plot(x,u,xc,points,'o'); 
    set(gca,'Fontsize',[fs])        
    legend('Solution','WOFD grid',3)        
    title(['Burgers'' equation (t = ', num2str(time), ')'],'Fontsize',fs)    
    xlabel('x');
    H=ylabel('u');
    set(H,'Rotation',0);
%    H = get(gca);
%    H.YLim(1) = -1;
    set(gca, 'YLim', [-1 1])
    figure(1)
    orient tall
    if saveplots
%     eval (['print -depsc2 burger' num2str(time) '_' num2str(rho) '.eps']);   
     eval (['print -deps burger' num2str(time) '_' num2str(rho) '.eps']);   
    end;  
    
    if ~redoing
      name = num2str(time);
%      figure(3); plot(x,u,xc,points,'o'); %title(name);
      if k==0
        scaling = axis;
      else
        axis(scaling)
      end;		
%      eval (['print -dgif8 ', 'Burger_', name, '.gif']); % Export fig 3
    end
    figure(2)
    figure(1)
      
    %---------------------------------------  
    % Update operators according to new grid
    %---------------------------------------      

    [Dxc,Dxxc] = updfdmat(Dxc,Dxxc,x,L,p,ind,added,removed);       

    
 %TEST ONLY
 % [Dxref,Dxxref] = fdmat(xc,L,p);  % Compute the full operators for reference       
 % fprintf('Error = %8.2e, %8.2e\n',max(max(abs(full(Dxc-Dxref)))), max(max(abs(full(Dxxc-Dxxref)))))   


    Ic = speye(Nc);           % New Crank Nicolson matrices
    Ac = Ic - 0.5*dt*nu*Dxxc;
    Ec = 2*Ic-Ac; 
    [Lowc, Upc] = lu(Ac);

    fprintf('.');
    fprintf('\n');
    k = 0;
    
    flops_eval = flops_eval + flops-f0;
    cpu_eval = cpu_eval + cputime-c0;
  end;

  %-------------------------
  % Solve on restricted grid
  %-------------------------
  f0=flops;
  c0=cputime;
  
  uxc = Dxc*uc;  
  tmpc = Ec*uc - dt*(mu*uc+rho).*uxc;   
  tmpc = Lowc\tmpc;
  uc   = Upc\tmpc;
  
  flops_solve=flops_solve+flops-f0; 
  cpu_solve=cpu_solve+cputime-c0; 
 
  %---------------------------
  % Solve on finest grid (ref)
  %---------------------------
  f0ref=flops;
  c0ref=cputime;
  
  ux = Dx*uref;  
  tmp = E*uref - dt*(mu*uref+rho).*ux;   
  tmp = Low\tmp;
  uref   = Up\tmp;

  flops_solve_ref=flops_solve_ref+flops-f0ref; 
  cpu_solve_ref=cpu_solve_ref+cputime-c0ref; 

  %------------------------------
  % Advance time and step counter
  %------------------------------
  time = time + dt;            
  k    = k+1;  
end;



%-------------------------------
% Reconstruct u on finest grid x
%-------------------------------    

aug=min(8,Nc);
xcP = [xc(Nc-aug+1:Nc)-L;xc;xc(1:aug)+L];
ucP = [uc(Nc-aug+1:Nc);uc;uc(1:aug)];    
u = interp1(xcP,ucP,x,interpol);

if any(isnan(u))
   fprintf('WARNING\n');
end

figure(2); plot(x,(uref-u)/max(abs(u))); title('Error','Fontsize',fs);
set(gca,'Fontsize',[fs])
if saveplots
  eval (['print -depsc2 burger' num2str(time) '_' num2str(rho) '_e.eps']);
end;  

err=max(abs(uref-u))/max(abs(u));
fprintf('%5.4f %12.8f \n',time,err)             



flops_eval      = flops_eval/1.0e6;
flops_solve     = flops_solve/1.0e6;
flops_solve_ref = flops_solve_ref/1.0e6;

fprintf('Eval        = %8.2f MFlop/s, %10d s\n',flops_eval,cpu_eval);
fprintf('Solve       = %8.2f MFlop/s, %10d s\n',flops_solve,cpu_solve);
fprintf('Total(E+S)  = %8.2f MFlop/s, %10d s\n',flops_solve+flops_eval,cpu_solve+cpu_eval)
fprintf('\n');
fprintf('Solve (ref) = %8.2f MFlop/s, %10d s\n',flops_solve_ref,cpu_solve_ref);


figure(3); clf; plot(x,u,xc,points,'o'); 
set(gca,'Fontsize',[fs])    
title(['Burgers equation WOFD (t = ', num2str(time), ')'],'Fontsize',fs)
xlabel('x');
H=ylabel('u');
set(H,'Rotation',0);    
legend('Solution','WOFD grid',3)    
if saveplots
  eval (['print -depsc2 burger' num2str(time) '_' num2str(rho) '.eps']);   
end;  




