function diftest(D)
%DIFTEST  Convergence test of differentiation matrix
% 
%   diftest(D)
%
%  Input
%    D:  wavelet genus 
%  
%
% Computes the differentiation error E^{(d)}(J) and
% estimate the convergence order by
%
%   log_2( E^{(d)}(J) ) - log_2( E^{(d)}(J+1) )
%
% as described on page 113 in "Wavelets in Scientific Computing". 

% OMN - 1998


%----------------
% Initialization
%----------------

if nargin < 1
  D = 6;	% Genus
end
L = 1;          % Domain
k = 4*pi;
minn  = 3;      % Lowest spatial resolution will be 1/2^(minn)
maxn  = 8;      % Highest spatial resolution will be 1/2^(maxn)

E1 = zeros(1,maxn);
E2 = zeros(1,maxn);

for n = minn:maxn
  N = 2^n;	    % Noof sample points
  fprintf('Computing error at resolution N = %d\n', N);
 

  h = 1/N;   
  x = (0:h:L-h)';  % N sample points (periodic)

  D1 = difmatrix(1,N,1,D); % Get differentiation matrix  1
  D2 = difmatrix(2,N,1,D); % Get differentiation matrix  2
  D3 = difmatrix(3,N,1,D); % Get differentiation matrix  3
  D4 = difmatrix(4,N,1,D); % Get differentiation matrix  4
  D5 = difmatrix(5,N,1,D); % Get differentiation matrix  5
  D6 = difmatrix(6,N,1,D); % Get differentiation matrix  6
  D2a = D1*D1;
  D3a = D1*D1*D1;
  D4a = D1*D1*D1*D1;
  D5a = D1*D1*D1*D1*D1;
  D6a = D1*D1*D1*D1*D1*D1;

  u      =      sin(k*x);
  du1_an =  k  *cos(k*x);
  du2_an = -k^2*sin(k*x); 
  du3_an = -k^3*cos(k*x);
  du4_an =  k^4*sin(k*x); 
  du5_an =  k^5*cos(k*x);
  du6_an = -k^6*sin(k*x); 
  
  du1  = D1*u;
  du2  = D2*u;
  du3  = D3*u;
  du4  = D4*u;
  du5  = D5*u;
  du6  = D6*u;
  du2a  = D2a*u;
  du3a  = D3a*u;
  du4a  = D4a*u;
  du5a  = D5a*u;
  du6a  = D6a*u;

  E1(n)=norm(du1_an-du1)/norm(du1);
  E2(n)=norm(du2_an-du2)/norm(du2);
  E3(n)=norm(du3_an-du3)/norm(du3);
  E4(n)=norm(du4_an-du4)/norm(du4);
  E5(n)=norm(du5_an-du5)/norm(du5);
  E6(n)=norm(du6_an-du6)/norm(du6);
  E2a(n)=norm(du2_an-du2a)/norm(du2a);
  E3a(n)=norm(du3_an-du3a)/norm(du3a);
  E4a(n)=norm(du4_an-du4a)/norm(du4a);
  E5a(n)=norm(du5_an-du5a)/norm(du5a);
  E6a(n)=norm(du6_an-du6a)/norm(du6a);

if 0    
  figure(1);
  subplot(6,2,1); title(num2str(N))
    plot(x,du1,x,du1_an);
    ylabel('du1');
    xlabel('x');
  subplot(6,2,3); 
   plot(x,du2,x,du2_an);
    ylabel('du2');
    xlabel('x');
  subplot(6,2,5); 
    plot(x,du3,x,du3_an);
    ylabel('du3');
    xlabel('x');
  subplot(6,2,7); 
    plot(x,du4,x,du4_an);
    ylabel('du4');
    xlabel('x');
  subplot(6,2,9); 
    plot(x,du5,x,du5_an);
    ylabel('du5');
    xlabel('x');
  subplot(6,2,11); 
    plot(x,du6,x,du6_an);
    ylabel('du6');
    xlabel('x');
  subplot(6,2,2);
    plot(x,zeros(size(x))); title(['N=' num2str(N) ', D=' num2str(D)]);     
  subplot(6,2,4); 
    plot(x,du2a,x,du2_an);
    ylabel('du2a');
    xlabel('x');
  subplot(6,2,6); 
    plot(x,du3a,x,du3_an);
    ylabel('du3a');
    xlabel('x');
  subplot(6,2,8); 
    plot(x,du4a,x,du4_an);
    ylabel('du4a');
    xlabel('x');
  subplot(6,2,10); 
    plot(x,du5a,x,du5_an);
    ylabel('du5a');
    xlabel('x');
  subplot(6,2,12); 
    plot(x,du6a,x,du6_an);
    ylabel('du6a');
    xlabel('x');
end    
end

%maxn=maxn-1;
%minn=maxn-1;
fprintf('D=%d\n',D);
fprintf('Convergence orders for different resolutions using D^{(d)}:\n')
fprintf('J:  '); disp([minn:maxn])
fprintf('d=1:'); disp(-diff(log2(E1(minn:maxn))));
fprintf('d=2:'); disp(-diff(log2(E2(minn:maxn))));
fprintf('d=3:'); disp(-diff(log2(E3(minn:maxn))));
fprintf('d=4:'); disp(-diff(log2(E4(minn:maxn))));
fprintf('d=5:'); disp(-diff(log2(E5(minn:maxn))));
fprintf('d=6:'); disp(-diff(log2(E6(minn:maxn))));
fprintf('Convergence orders for different resolutions using D^d:\n')
fprintf('J:  '); disp([minn:maxn])
fprintf('d=1:'); disp(-diff(log2(E1(minn:maxn))));
fprintf('d=2:'); disp(-diff(log2(E2a(minn:maxn))));
fprintf('d=3:'); disp(-diff(log2(E3a(minn:maxn))));
fprintf('d=4:'); disp(-diff(log2(E4a(minn:maxn))));
fprintf('d=5:'); disp(-diff(log2(E5a(minn:maxn))));
fprintf('d=6:'); disp(-diff(log2(E6a(minn:maxn))));


%log2(E1(minn:maxn))
%log2(E2(minn:maxn))
figure(2); clf
v = minn:maxn;
plot(v,log2(E1(v)),v,log2(E2(v)),v,log2(E3(v)),v,log2(E4(v)),v,log2(E5(v)),v,log2(E6(v)))
legend('1','2','3','4','5','6')
title(['Convergence rates D=' num2str(D)])

figure(3); clf
v = minn:maxn;
plot(v,log2(E1(v)),v,log2(E2a(v)),v,log2(E3a(v)),v,log2(E4a(v)),v,log2(E5a(v)),v,log2(E6a(v)))
title(['Alternative convergence rates D=' num2str(D)])
legend('1','2','3','4','5','6')





