function T=dstmat(wavelet,r,j,q,K,k)
%DSTMAT  Get matrix T with scaling function values.
%
%   T=dstmat(D,r,j,q,K,k)
%
%  Input
%    D:  Wavelet genus - optional, default D=4.
%    r:  Dyadic resolution of function.
%    j:  Dyadic resolution of scaling function coefficients.
%    q:  Dyadic resolution of scaling function.
%    K:  Optional. Integer (see below). Default K=1
%    k:  Optional. Index vector indicating which rows to be computed.
%                  Default k=0 - all rows are computed.
%
%  Output
%    T:  Matrix transforming between a function f in V_r
%        and a vector of scaling function coefficients c.
%   
%  T will have K*2^r rows and K*2^j columns.
%
%  The following must hold:  j+q-r >= 0 
%
%  Example:
%
%    T = dstmat(4,3,3,0,1)
%

% Ole Moller Nielsen, DTU - September 1996.

if nargin < 6
  k = 0;        % 0 indicates all rows.
  if nargin < 5
    K = 1;      % Factor which is not a positive pwroftwo (integer)
  end;
end;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Parameter check
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

len = length(wavelet);  %Determine the nature of argument #1:
if len > 1
  D  = len;             % Use actual filter length
  lp = wavelet;         % First argument is the wavelet filter (from filters.m)
else
  D  = wavelet;         % First argument is the wavelet genus
  lp = daubfilt(D);     % Low pass filter    
end;    

shift=1;

if j+q-r < 0 
  error('Resolution of phi is too coarse. Adjust j, q, or p so j+q-r >= 0');
end;

[x,phi] = cascade(lp,q); % Get phi:		

N=K*2^r;    % Number of samples
P=K*2^j;    % Number of coefficients

Q=2^q;      % The resolution of phi
            % (D-1)*Q is the number of dyadic points where phi is known.
S = P*Q/N;  % step between values needed in phi is 2^(j-r+q) 

%%%%%%%%%%% Compute all rows %%%%%%%%%%%%	    
if k==0,
  T = spalloc (N,P,(D-1)*2^(r-j)); % There will be (D-1)*2^(r-j) entries 
				   % in each column
  phi_s  = phi(1:S:length(phi));
  philen = length(phi_s);
  
  if philen < N                % phi_s fits into a column
    for l=0:P-1
      firstrow = l*N/P;
      lastrow  = firstrow + philen - 1;
      wrap = lastrow - (N-1);
      if wrap > 0, 
	lastrow = lastrow - wrap;
	T(0+shift:wrap-1+shift, l+shift) = phi_s(philen-wrap+1:philen);     
      else 
	wrap=0; 
      end;
      T(firstrow+shift:lastrow+shift, l+shift) = phi_s(1:philen-wrap);
    end;  
  else     % RARE (If vectors are small): 
	   % phi_s does not fit into a column (use modulus wrapping)
    for l=0:P-1   % For each column
      firstrow = rem(l*N/P,N);
      for k=0:philen-1                % For element in phi_s
	row = rem(firstrow + k, N);
	T(row+shift, l+shift) = T(row+shift, l+shift) + phi_s(k+shift);     
      end;
    end;
  end;      
  
else
  %Compute only rows selected by the index vektor k

  % Generate matrix for interpolating function values
  % at the points k/2^p given equidistant scaling function 
  % coefficients at resolution 2^j, where p>j. 
  % The wavelet genus is given by D. 
  
  L = length(k); 
  T = spalloc (L,P,(D-1)*2^(r-j)); % There will be (D-1)*2^(r-j) entries   
  
  PQ = P*Q;
  for i = 1:L
    m1 = (k(i)-1)*S;
    l=0;
    while l<P           % loop over columns     IMPROVE!!!
%      mt = m1-l*Q;  
      m = pmod(m1-l*Q,PQ)+1;  %Improve
%      [mt m]

      
      if m>0 & m <= length(phi)
        T(i,l+1) = phi(m);
      end;	
      l=l+1;
    end;
  end;  
 
end;

T=2^(j/2)*T;





