function [xext,yext]=extend(x,y,D,lambda,exttype)
%[xext,yext]=extend(x,y,D,lambda,exttype)
%
% Compute Smooth Wavelet Continuation for y.
%
% Input
%   x: Abscissae (column vector)
%   y: Signal(s) to be extended (column vectors)
%   D: Wavelet genus
%   lambda: Transform depth
%   exttype: Type of extension
%     0: Periodic
%     1: Reflection
%     2: Polynomial
%     3: Smooth Wavelet Continuation (SWC)
%     4: Combined ? (not yet implemented)
%
% Output
%   xext: Extended abscissae
%   yext: Extended signal
%
%
% OMN, ANU - January 1999


if nargin < 5
  exttype = 1
end;

enable_caching = 1; %Use of cached matrices (exttype==3 only) 
verbose = 1;


%
% Data check
%
[M,N] = size(y);
if length(x) ~= M
  fprintf('ERROR (extend): Mismatch, size(x)=[%d,%d], size(y)=[%d,%d].\n',...
  size(x,1), size(x,2),M,N);
  xext = [];
  yext = [];
  return;
end;

%
% Determine y1, the extension of y
%
if D <= 2
  S  =  0;
  xext = x;
  y1 = [];  
else    
  if exttype == 0       %Periodic
    S = 0;
    y1 = [];   
    str = 'periodization';
  elseif exttype == 1   %Reflection
    S  = M;
    y1 = flipud(y); 
    str = 'reflection';    
  elseif exttype == 2   %Polynomial
    S  = 2^lambda*(D+2);
    str = 'polynomial continuation';         
  elseif exttype == 3   %Smooth Wavelet Continuation
    D22 = (D-2)/2;
    S  = lambda*2^lambda*D22; 
    str = 'wavelet continuation';             
  else
    fprintf(['WARNING (extend.m): Unknown extension = %d,',...
             'using periodization'],exttype);  
    [xext,yext]=extend(x,y,D,lambda,0);	     
    return
  end;     

  %S is length of extension    
  MS = M+S;
  L = x(end) + x(2)-x(1);     %Old interval endpoint  
  xext = [x;(0:S-1)'/M+L];
  L = xext(end) + x(2)-x(1);  %New interval endpoint

  if verbose & exttype > 0, 
    fprintf('Original length %d extended with %d points by %s\n',M, S, str);
  end;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Polynomial extension
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
  if exttype == 2
    %
    %Extend right end of interval
    %
    P = D/2; 
    
    %
    %Fit polynomial of degree P-1 to last P points     
    %
    X = ones(P,1);
    xp = x(end-P+1:end);    %Last P points of x
    yp = y(end-P+1:end,:);  %Last P rows of y  
    for k=1:P-1
      X(:,k+1) = X(:,k).*xp;  
    end;
    warning off
    gamma = X\yp;  %Solve for all columns
    warning on
    
    %
    % Extrapolate halfway to the right
    %
    X = ones(S/2,1);
    xp = xext(end-S+1:end-S/2);  %First S/2 points of extension
    for k=1:P-1
      X(:,k+1) = X(:,k).*xp;  
    end;
    yr = X*gamma; 
    
    
    %
    %Extend left (wrapped) end of interval
    %
  
    %
    %Fit polynomial of degree P-1 to first P points of x    
    %
    X = ones(P,1);
    xp = x(1:P);       %First P points of x
    yp = y(1:P,:);     %First P rows of y  
    for k=1:P-1
      X(:,k+1) = X(:,k).*xp;  
    end;
    warning off
    gamma = X\yp;
    warning on
    
    %
    % Extrapolate halfway to the left
    %
    X = ones(S/2,1);
    xp = xext(end-S/2+1:end)-L;  %Last S/2 points of extension
    for k=1:P-1
      X(:,k+1) = X(:,k).*xp;  
    end;
    yl = X*gamma; 
  
    y1 = [yr;yl];
    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Smooth wavelet extension
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
  elseif exttype == 3  
    if M < S/lambda 
      fprintf(['WARNING: Very few data points (M=%d) for D=%d, lambda=%d.'... 
      ' M should be greater than or equal to S/lambda=%d\n'],M,D,lambda,S/lambda);
    end;

    %
    %Check for precomputed system in directory ~/.wapa_cache/
    %
    homepath = getenv('HOME');
    CPN = [homepath '/.wapa_cache/'];     
    if ~exist(CPN), eval(['!mkdir ' CPN]); end;
    CFN = ['ext_M=' int2str(M) '_D=' int2str(D) '_d=' int2str(lambda)];
    FFN = [CPN CFN]; 
  
    if exist([FFN '.mat.gz']) & enable_caching
      %if verbose, fprintf('FOUND cached file %s - making a copy\n', CFN); end;
      if verbose, fprintf('FOUND cached file %s\n', CFN); end;
      eval(['!/bin/cp ' FFN '.mat.gz ' FFN '.tmp.gz']);  %Copy compr file
      %if verbose, fprintf('Uncompressing - '); end;
      eval(['!gunzip -f ' FFN '.tmp.gz']);            %Uncompress
      %fprintf('Moving - ');  
      eval(['!/bin/mv -f ' FFN '.tmp ' FFN '.mat']);  %Rename file   
      %fprintf('Loading into Matlab\n');    
      eval(['load ' FFN]);                            %Load it 
      %fprintf('Removing uncompressed file from disk (background)\n');      
      eval(['!/bin/rm ' FFN '.mat &']);               %Remove uncompr file     
    else
      %if verbose, 
      %  fprintf('Did NOT find cached file %s - computing new system\n', CFN);
      %end;
      
      %
      % Compute new matrixes B and C
      % 
      for i = 1:lambda
  	Si      = S/2^i + D22;  %Number of eqs related to scale i;
  	Bsub{i} = sparse(Si,S); %The i'th submatrix
      end;
      
      %Build linear system to determine extension using highpass coefficients
      lp = daubfilt(D);  %Lowpass coefficients
      hp = low2hi(lp);   %Highpass coefficients
    
      %
      % Construct Bi starting with the coarse-scale interactions 
      %
      for i=lambda:-1:1;
  	[Si, dummy] = size(Bsub{i});
  	filters = zeros(D^i,1); % Temporary storage for pre-computed filters 
  	indvec = zeros(D^i,1);  % Temporary storage for indices    
  	Btmp = sparse(Si,S);    % Temporary storage for coefficient matrix
  	Ctmp = sparse(Si,M);    % Temporary storage for computing rhs    
    
  	    
  	if verbose 
  	  fprintf('Construct equations for scale %d (Si=%d)\n',i,Si);
  	end;	 
  	
  	lstart = M/2^i-D22;
  	lstop  = MS/2^i-1;
  	for l=lstart:lstop,
  	  row = l-lstart+1;
	  if verbose
  	    if rem(row,ceil(Si/10))==0, fprintf('%d ',row); end;
	  end;  
  	  
  	  if l==lstart          %Do all possible precomputations
  	    k = zeros(i,1); 
  	    for n=0:D^i-1;
  	      %
  	      % Compute index into extended vector 2^i+sum_{m=1}^i 2^{j-1} k_m
  	      %
  	      indsum = 2*l + k(i);
  	      for m=i-1:-1:1, indsum = 2*indsum+k(m); end;
	      indvec(n+1) = indsum;
	      
  	      %
  	      % Compute relevant product of filter coefficients for level i
  	      %
  	      fc = hp(k(i)+1);
  	      for m=i-1:-1:1, fc = fc * lp(k(m)+1); end;   
	      filters(n+1) = fc;   
	      
  	      %
  	      % Increase k(1) by one and let carry ripple left 
  	      %    
  	      m=1;
  	      k(m) = k(m) + 1;      
  	      while k(m) == D & m < i,
  		k(m) = 0;
		m = m+1;
  		k(m) = k(m) + 1;      	
  	      end;
  	    end;
  	  else
  	    indvec = indvec + 2^i; %Displace indices as l advances
  	  end;
    
  	  %
  	  % Compute all elements in linear system
  	  %      	
  	  for n=0:D^i-1
	    %
  	    % Get index into extended vector
  	    %
  	    indsum = indvec(n+1);
	    
  	    %Compute index = rem(indsum,MS);  	  
	    if indsum >= 0 & indsum < MS   
	      index = indsum;
	    elseif indsum >= MS & indsum < 2*MS
	      index = indsum - MS; %indsum will never exceed 2*MS
	    else
  	      index = pmod(indsum,MS);  	  %The general but slow formula
	    end;    
    
  	    %
  	    % Decide whether element goes into coefficient matrix B or 
	    % matrix C used to compute right hand side
  	    %
  	    if index >= M      %These are unknown so filters go into B
	      col = index-M+1;  
	      Btmp(row,col) = Btmp(row,col) + filters(n+1);
	      
  	    else               %These are known so they go into C	
	      Ctmp(row,index+1) = Ctmp(row,index+1) + filters(n+1); 
  	    end;
  	  end;  %for n=0:D^i-1
  	end;
  	
  	Bsub{i} = Btmp;
  	Csub{i} = -Ctmp; %Change sign because these elements belong to right hand side
  	
  	if verbose, fprintf('\n'); end; 
      end;
    
      
      %fprintf('Assemble\n');
      C = [];  
      B = [];
      for i = 1:lambda  
  	C = [Csub{i};C];            
  	B = [Bsub{i};B];
      end;
  
      if verbose, fprintf('Save computed system as %s\n', CFN); end;   
      eval(['save ' FFN ' B C']);        %Save B and C
      eval(['!gzip -f ' FFN '.mat &']);  %Compress in background 
    end;
    
    
    b = C*y;  %Compute right hand sides;
    
    %
    % Compute Laplacian for penalized solution
    % 
    h       = 1/S;        % grid spacing
    Dk      = sparse(toeplitz([2/h,-1/h,zeros(1,S-2)]));
    Dk(1,1) = Dk(1,1)/2;
    Dk(S,S) = Dk(S,S)/2;
    
    alpha = eps;
    
%    A = full(B)+alpha*Dk;  %Bloody sensitive to full versus sparse!!!
%    A = full(B);  %Bloody sensitive to full versus sparse!!!
    A = B+alpha*Dk; 
  
    if lambda > 1          %We require zero mean of extension
      A = [A; ones(1,S)/S];
      b = [b; zeros(1,N)];
    end;
    
    %fprintf('Solve\n');                
    warning off;
    y1 = A\b;     %Solve for all right hand sides
    warning on;
    %[L,U,P]=lu(A);
    %y1 = U\(L\P*rhs);
  
%    r = b-A*y1;
%    [m1,m2]=max(abs(b-A*y1))
      
    
  end;  
end;


%
% Extension beyond [0,1[
%
%%%%xext = [x;(0:S-1)'/M+1];
yext = [y;y1];

return




