/*
 * imfwt.c - inverse multiple rowwise wavelet transform for use in MATLAB 
 * e.g. by the inverse 2d wavelet transform (ifwt2.m). 
 *
 * This is a MEX-file for MATLAB optimized for speed as follows
 *   - Input and output workspaces are alternated to reduce copying.
 *   - Inner loops address data consecutively to make good use of cache.
 *   - Loops that may cause wrapping are separated into fast 'bulk' loops
 *     that don't wrap and special cases where wrapping may occur.
 *
 * When compiled (using mex imfwt.c) this routine can be called from
 * MATLAB as follows:
 *
 *   Y = imfwt(X, lambda, lp, hp);  
 *
 * where
 *   X is an M x N matrix
 *   lambda is the number of desired transform steps
 *   lp is a wavelet low-pass filter 
 *   hp is the corresponding high pass filter (e.g. [lp,hp]=daubfilt(8))
 *
 *   Y is a matrix of same size as X containing the inverse 
 *   wavelet transform of all rows of X.
 * 
 * NB! The input matrix X is used as workspace and will be destroyed.
 *     It is the responsibility of the caller to make a copy if necessary.
 *
 *
 * Ole Nielsen, ANU, 1999
 *
 */

	

#include "mex.h"
#include "math.h"



/*********************************************************************/
/************ ONE STEP OF THE INVERSE WAVELET TRANSFORM **************/
/*********************************************************************/
/*
 * impwt performs one step of the inverse wavelet transform of length N on 
 * all ROWS of the MxN matrix X using filters lp and hp each of length D.
 * The result is returned in the first N columns of Y
 *
 * Mathematically, Y(:,1:N) = X(:,1:N) * W, 
 * where W has the form (shown here for N=8 and D=4) 
 *
 *   | lp[0] lp[1] lp[2] lp[3]                          |
 *   |             lp[0] lp[1] lp[2] lp[3]              |
 *   |                         lp[0] lp[1] lp[2] lp[3]  |
 *   | lp[2] lp[3]                         lp[0] lp[1]  |  
 *   | hp[0] hp[1] hp[2] hp[3]                          |
 *   |             hp[0] hp[1] hp[2] hp[3]              |
 *   |                         hp[0] hp[1] hp[2] hp[3]  |
 *   | hp[2] hp[3]                         hp[0] hp[1]  |  
 * 
 *
 *
 * Ole Nielsen, ANU, 1999 
 *
 */

int impwt (double *X, int M, int N, double *lp, double *hp, int D, double *Y)
{
  int i,j,k,l,row, NH,r,r1,NH1,NHM,M2,crossover,mul,NH_multiple;
  int index,indexi,indexj,indexh,index1,indexr,kstart;

  /*mexPrintf("impwt: N = %d\n", N);*/

  
  NH  = N/2;
  NHM = NH*M;
  M2 = 2*M;
  
  kstart = 2;  /*2: takes special care of two first coefs, 0 don't */ 
  
  mul = (int) (D-2)/2/NH+1;
  NH_multiple = mul * NH; /* Nearest multiple of NH > (k-i)/2 forall k,i */
  
  if (kstart < 2) {
    for (i = 0; i < N; i++) {
      index = i*M;      
      for (row = 0; row < M; row++) {
        Y[index] = 0.0;   /* Reset result matrix */
        index++;
      }
    }
  } else {
  
    /* Take special care of the two first filter coefficients */
    k = 0;
    l = 1;
    
    r = 0;
    indexi = 0;
    indexr = 0;
    for (i = 0; i < N-1; i=i+2) {
      indexi = i*M; 
      indexr = r*M; 
            
      index1 = indexi + M;      
      indexh = indexr + NHM;    
      for (row = 0; row < M; row++) { 
        Y[indexi]= lp[k]*X[indexr] + hp[k]*X[indexh];
        Y[index1]= lp[l]*X[indexr] + hp[l]*X[indexh];
	indexi++;
	indexr++;
	index1++;
	indexh++;
      }
      /*indexi = indexi + M2;*/
      /*	indexr = indexr + M;    */
      r  = r+1;   /* r  = (i-k)/2  */
    }         
  }
  
  
  for (k=kstart; k<D-1; k=k+2) {        
    if (k > N) {      /* Separate those i, where i-k<0 */
      crossover = N;
    } else {
      crossover = k; 
    }  
        
    l = k+1;
                   
    /*--- Modulus phase  - only a fraction of the total time is spent here */ 

    for(i=0; i<crossover-1; i=i+2) {
      j  = (i-k)/2;                    
      r  = (NH_multiple + j) % NH;          
      
      /*      mexPrintf("%d %d %d %d\n",k,i,j,r); */
      
      indexi = i*M;      
      index1 = indexi+M;
      indexr = r*M;
      indexh = (r+NH)*M;
      for (row = 0; row < M; row++) {
        Y[indexi] = Y[indexi] + lp[k]*X[indexr] + hp[k]*X[indexh];
        Y[index1] = Y[index1] + lp[l]*X[indexr] + hp[l]*X[indexh];   
	indexi++;
	indexr++;
	index1++;
	indexh++;
      }
    }     

    /* Linear phase - Most of the time is spent here */
    r = 0;
    for (i = crossover; i < N-1; i=i+2) {
      indexi = i*M;
      index1 = indexi+M;      
      indexr = r*M;
      indexh = (r+NH)*M;            
      for (row = 0; row < M; row++) { 
        Y[indexi]=Y[indexi] + lp[k]*X[indexr] + hp[k]*X[indexh];
        Y[index1]=Y[index1] + lp[l]*X[indexr] + hp[l]*X[indexh];
	indexi++;
	indexr++;
	index1++;
	indexh++;
      }
      r  = r+1;   /* r  = (i-k)/2  */
    }     
  }  
}


/*****************************************/
/* MATLAB GATEWAY ROUTINE AND OUTER LOOP */
/*****************************************/
/*
 * This is the interface between MATLAB and C
 *
 * Parameters are extracted and the outer loop of the
 * inverse wavelet transform is carried out here. 
 *
 */
 
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
  int i, j, k, M, N, NH, N2, index, D, lambda, Coarsest_level, pwr;
  int dir, row, FinalN;
  double *X, *Y, *lp, *hp;
  
  /****************************/  
  /* A very basic input check */
  /****************************/
  if ( nrhs != 4 )
    mexErrMsgTxt("Four inputs required!");
  else if ( nlhs > 1 )
    mexErrMsgTxt("Too many output arguments!");  
    

  /*******************************************/
  /* Get input matrix from MATLAB argument 1 */
  /*******************************************/
  M = mxGetM(prhs[0]);   /* Number of rows         */
  N = mxGetN(prhs[0]);   /* Number of columns      */
  X = mxGetPr(prhs[0]);	 /* The input M x N matrix */			 

  
  /***************************************************************/  
  /* Get wavelet transform depth, lambda, from MATLAB argument 2 */
  /***************************************************************/
  lambda = (int) mxGetScalar(prhs[1]);
  if (mxIsInf(mxGetScalar(prhs[1])))
  {                          /* If requested depth == Inf then */         
      Coarsest_level = 1;    /* take as many steps as possible */
  } else
  {
    pwr = pow(2,lambda);      
    Coarsest_level = max((int) (N/ (float) pwr), 1);      
  }

  /****************************************************/
  /* Get wavelet filters from MATLAB argument 3 and 4 */
  /****************************************************/  
  D = max(mxGetM(prhs[2]),mxGetN(prhs[2]));   /* Filter length */
  if (D != max(mxGetM(prhs[3]),mxGetN(prhs[3]))) 
    mexErrMsgTxt("Filters must have same length. Pad with zeros!");      
  
  lp = mxGetPr(prhs[2]); /* Low pass filter  */
  hp = mxGetPr(prhs[3]); /* High pass filter */
  
  /************************************************************/
  /* Create output array. This will also be used as workspace */
  /************************************************************/  
  plhs[0] = mxCreateDoubleMatrix(M, N, mxREAL);
  Y = mxGetPr(plhs[0]);	   
  	

  /***********************************************/	
  /* Outer loop of the inverse wavelet transform */
  /***********************************************/	

  /* Find initial transform size */
  FinalN = N;   
  dir = 0;          /* Alternate transform direction to save space */    
  while ( (N%2) == 0 & N > Coarsest_level ){
    dir = 1-dir;  
    N=N/2;
  }
  NH = N;

  if (dir == 0){                /* Begin transform in Y */
    for (k = 0; k < NH; k++) {  /* Move dc terms to Y */
      index = k*M;      
      for (row = 0; row < M; row++) {
    	Y[index] = X[index];  
    	index++;
      }    
    } 
  }
  
  
  /*Inverse pyramid algorithm starting with finest scale*/
  while (NH < FinalN){  
    N  = 2*NH;  
  
    if (dir == 1) {    
      impwt (X, M, N, lp, hp, D, Y);  /* X -> Y */      
    } else {  
      for (k = NH; k < N; k++) {  /* Move differences from X to Y */
        index = k*M;      
        for (row = 0; row < M; row++) {
    	  Y[index] = X[index];  
    	  index++;
        }
      }
      impwt (Y, M, N, lp, hp, D, X);  /* Y -> X */            	    
    } 
    NH  = N;
    dir = 1-dir;
  }  
} /* END MATLAB GATEWAY ROUTINE */		 


/***********************************************************/
/****************** MISC AUXILIARY ROUTINES ****************/
/***********************************************************/
int max(int a, int b)
{
  if (a > b)
    return(a);
  else
    return(b);  
}




