function X = imfwt(X, depth, lp, hp)
%IFWT Inverse Multiple Fast Discrete Periodized Wavelet Transform.
%
%  X = imfwt(X, lambda, lp, hp)
%
%  Input
%    X:      Matrix, where ROWS are the vectors to be transformed.
%    lambda: Depth of the transform - optional.
%            Default is largest possible depth.            
%    lp, hp: Filters - default [lp,hp] = daubfilt(4);
%
%  Output
%    X:  Matrix, where ROWS are the transformed vectors
%
%  The number of columns in X should have a power of 2 as a factor. 
%
%  See also mfwt.

% Ole Moller Nielsen - 1998.


%%%%%%%%%%%%%%% Parameter check
if nargin < 4
  if nargin == 3, 
    hp = low2hi(lp); 
  else    
    [lp, hp] = daubfilt(4); % Default filters   
    if nargin < 2
      depth = inf;  
    end;  
  end;
end;


D   = length (lp);
if D ~= length(hp);
  fprintf('ERROR (mfwt): Filters must be of same length. Pad shortest one with zeros.');
end;

[M, N]  = size (X);   % N is the length of each row

if M < 1 | N < 1, 
  fprintf('ERROR (imfwt.m): Zero length vector specified'); 
  X = [];
  return
end;

if N < 2, 
  fprintf('WARNING (imfwt.m): Length 1 column vector, no transform made'); 
end;


%%%%%%%%%%%%%%% End parameter check


Coarsest_level = max(N/2^depth, 1);

FinalN = N;              % Factor N=K*2^m
while rem(N,2)==0,       % Get smallest even N
  N=N/2;
end;  
N=max(N, Coarsest_level);
N=2*N;

%N=2;
while N <= FinalN    % Inverse pyramid algorithm starting with finest scale
  [X] = impwt(X,N,lp,hp);
  N   = 2*N;
end;




















