function X = mfwt(X,depth,lp,hp)
%FWT Multiple Fast Discrete Periodized Wavelet Transform.
%
%  X = mfwt(X,lambda,lp,hp)
%
%  Input
%    X:      Matrix, where ROWS are the vectors to be transformed.
%    lambda: Depth of the transform - optional.
%            Default is largest possible depth.            
%    lp, hp: Filters - default [lp,hp] = daubfilt(4);
%
%  Output
%    X:  Matrix, where ROWS are the transformed vectors.
%
%  The number of columns in X should have a power of 2 as a factor. 
%
%  This routine addresses columnwise which is more efficient
%  than rowwise in Matlab. 
%
%  See also imfwt, mpwt, impwt.

% Ole Moller Nielsen - 1998.

%%%%%%%%%%%%%%% Parameter check
if nargin < 4
  if nargin == 3, 
    hp = low2hi(lp); 
  else    
    [lp, hp] = daubfilt(4); % Default filters   
    if nargin < 2
      depth = inf;  
    end;  
  end;
end;


D   = length (lp);
if D ~= length(hp);
  fprintf('ERROR (mfwt): Filters must be of same length. Pad shortest one with zeros.');
end;

[M, N]  = size (X);   % N is Length of each row

if M < 1 | N < 1, 
  fprintf('ERROR (mfwt.m): Zero length vector specified'); 
  X = [];
  return
end;

if N < 2, 
  fprintf('WARNING (mfwt.m): Length 1 row vector, no transform made'); 
end;
  
%%%%%%%%%%%%%%% End parameter check

Coarsest_level = max(N/2^depth, 1);

while rem(N,2) == 0 & N > Coarsest_level;   % Allow for N=2^m*K
  [X,NH] = mpwt (X, N, lp, hp);
  N      = NH;                    % Reduce N to N/2 for next level
end;



















