function [X,NH] = mpwt(X, N, lp, hp)
%MPWT Multiple Partial (Single stage) Wavelet Transform.
%
%  [X, NH] = mpwt (X, N, lp, hp);
%
%  Input
%    X:      Matrix, where ROWS are the vectors to be transformed.
%    N:      Length of transform.
%    lp:     Low pass filter    
%    hp:     High pass filter (e.g. [lp,hp] = daubfilt(8))
%
%  Output
%    X:  Matrix, where ROWS are the transformed vectors.
%    NH: N/2.
% 
%  Performs one step of the wavelet transform 
%  on the first N components of each row of X.
%
% See also mfwt, imfwt, and impwt
%

% Ole Moller Nielsen - 1998.

% The algorithm is formulated as vector operations 
% on columns to increase efficiency.

D   = length (lp);
if D ~= length(hp);
  fprintf('ERROR (mpwt): Filters should be of same length. Pad shortest one with zeros.');
end;

%D   = length (wl)/2; % Wavelet genus
%lp  = wl(1:D);       % Low pass filter
%hp  = wl(D+1:2*D);   % High pass filter

T   = zeros (size(X));  % Workspace

NH  = N/2;
NH1 = NH+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take care of coefficients 1 and 2 first
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:NH                    % Column number in first half of result
  r1     = 2*i;               % Column number in X corresp to k=2
  r      = r1-1;              % Column number in X corresp to k=1
  j      = i+NH;              % Column number in second half of result
  T(:,i) = T(:,i) + lp(1)*X(:,r) + lp(2)*X(:,r1);
  T(:,j) = T(:,j) + hp(1)*X(:,r) + hp(2)*X(:,r1);
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% coefficients with k > 2 now
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

crossover = (D-2)/2;  % Index r will wrap when i > N/2-crossover


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute part that does not wrap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:NH-crossover            % Row number in first half of result
  for k=3:D                     % Wavelet coefficients
    r      = k+2*i-2;           % Row number in X
    j      = i+NH;              % Row number in second half of result
    T(:,i) = T(:,i) + lp(k)*X(:,r);
    T(:,j) = T(:,j) + hp(k)*X(:,r);
  end;  
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute wrapping part - only a fraction is spent here
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=max(NH - crossover + 1,1):NH
  for k=3:D                         % Wavelet coefficients
    r     = rem(k+2*i-3, N) + 1;   
    j     = i+NH;              
    T(:,i) = T(:,i) + lp(k)*X(:,r);
    T(:,j) = T(:,j) + hp(k)*X(:,r);
  end;
end;   

X(:,1:N) = T(:,1:N);  % Copy wavelet transform at level N back into X
clear T;
































