function nls
% Solve the nonlinear Schroedinger equation using using
% the FFT split-step method
%
%  i Ut = 0.5*beta2*Uxx - i*0.5*alpha*U - gamma*|U|^2*U 
%
%
%  U( -L/2, t) = U(L/2, t)
%  U( x, 0) = U_0
%
%
%  OMN - 1997
%

%------------------
% Method parameters
%------------------
analytic = 0;

epsilon = 1.0e-15;
slowiter = 25;
maxiter = 50;

%------------------
% Model parameters
%------------------
n                = 4;                         % 4 if p = 5
%n                = 2,3;                      % 3 if p = 7
Ld               = 64;                        % Domain size             
N                = Ld*2^n;                    % Grid resolution 

if analytic
  beta2            =-2;                   
  gamma            = 2;                       % Non-linearity
  alpha            = 0;                       % Loss
else
  beta2            =-2;                   
  gamma            = 2;                       % Non-linearity
  alpha            = 0;                       % Loss
end;

T                = 2^10;
finaltime        = 0.5;                       %
dt               = 1.0/T;                     % Timestep
time             = 0.0;                       % Simulated time 
plotinterval     = finaltime/8;              % Time between plots
stepsbetweenplot = round(plotinterval/dt);


%----------------------
% Abscissae: x in [-L/2:L/2]
%----------------------
h = Ld/N;   
x = (-Ld/2:h:Ld/2-h)';  % N sample points 

do     = 1/Ld;                     % Frequency interval. 
Omega  = (-N/2/Ld:do:N/2/Ld-do)';  % Frequency-vector (Abscissae). 


%%%%%%% Initial condition:

if analytic
  eta = 1;
  xi  = 1;
  u = eta*sech(eta*x).*exp(i*xi*x);    %initial condition (Corresp to analytic)
else
  u = 2 * sech(x);
end;

L = i*0.5*beta2*(2*pi*Omega).^2-alpha/2; % Linear operator in Fourier domain
E = fftshift(exp(dt/2*L));               % Precompute exponential (half timestep)

fprintf('N=%d, interval = %6.2f (%d steps), %s\n',N,plotinterval,stepsbetweenplot,'FFT');

%%%%%%% Predict solution for each time step.
%count = 1;
k     = 0;

% Nonlinear part 
usqr = abs(u).^2;
N1   = i*gamma*usqr;

flops_tot = 0;
f0        = flops;
cpu_tot   = 0;
c0        = cputime;
while time < finaltime,

  if k==0 | k==stepsbetweenplot
    figure(1); clf; plot(x,abs(u).^2);
%    title('FFT-SPLIT STEP');
    title('NLS (SPLIT STEP)');
    xlabel('x');
    ylabel('|u|^2');    
    
    fprintf ('time = %6.3f, Norm(u) = %f, Loss = %f\n',time,norm(u)*sqrt(h),alpha); 
    k = 0;
  end

  %%% FFT half timestep
  u0 = u;                          % Value at z
  u1 = u0;                         % Value at z+dz (approx)
      
  U0  = fft(u0);          % Fourier transform of u.
  Uhalf = E.*U0;                    % Apply linear operator
  uhalf  = ifft(Uhalf);   % Transform back to the spatial domain.
  
  % Iterate
  iter  = 0;
  error = 1;  
  N0 = N1;     % Use nonlinear part from the end of previous step    
  while error > epsilon
    u1old = u1;
  
    % Nonlinear part at t + dt (approx)  
    u1sqr = abs(u1).^2;       
    N1   = i*gamma*u1sqr;

    Q    = (N0 + N1) * dt/2;       % Trapez formula
    u_nl = exp(Q).*uhalf;          % Apply non-linearity
    
    U_nl = fft(u_nl);    % Fourier transform of u_nl.
    U1   = E.*U_nl;                % Apply linear operator
    u1   = ifft(U1);     % Transform back to the spatial domain

    error = norm(u1-u1old,inf)/norm(u1,inf);
    iter = iter + 1;
    if iter > slowiter
      fprintf('Warning - Slow convergence, iteration # = %d\n',iter);   
      if iter > maxiter
        fprintf('Split step does not converge\n');
        return
      end;
    end;         
  end;  
%  fprintf('Err = %e, Iterations=%d\n',error,iter);
  u = u1; 
 
  %------------------------------
  % Advance time and step counter
  %------------------------------
  time = time + dt;            
  k    = k+1;  
end %while  

flops_tot=(flops-f0)/1.0e6; 
cpu_tot = (cputime-c0);
fprintf('MFlops = %8.2f, time = %10d\n',flops_tot,cpu_tot);

if analytic
  t = x;
  z = time;
  % Analytic solution (periodized)
  theta = t - 2*xi*z;
    
  periodized = 0;          % Periodized
  while ~periodized    
    r = (theta < -Ld/2);    %Identify elements left of -L/2
    if sum(r) == 0
      periodized = 1;      % TRUE
    else
      theta = theta + Ld*r; %Periodize	
    end;
  end;
    
  sigma = (eta^2 + xi^2)*z;
  uanal = eta*sech(eta*theta).*exp(i*(xi*theta+sigma));

  error = max(max(abs(u-uanal)))/max(max(abs(u)));
  fprintf('error = %8.2e, log2(error) = %e\n',error,log(error)/log(2));

  figure(1);
  plot (t,abs(u),t,abs(uanal),'--');    
else  
%  figure(1)
%  plot (t,abs(u));
%  ylabel('|u|') 
end;  
%if(firstplot), ax=axis; firstplot=0; else axis(ax); end;
%fprintf ('z = %6.3f, Norm(u) = %f, Loss = %f\n',z,norm(u)*sqrt(h),alpha); 







