function nls(tol,refsol)
%function nls(tol,refsol)
%
% Solve the nonlinear Schroedinger equation 
%
%  i Ut = 0.5*beta2*Uxx - i*0.5*alpha*U - gamma*|U|^2*U 
%
%
%  U( -L/2, t) = U(L/2, t)
%  U( x, 0) = U_0
%
%
%  Irregular grid generated using wavelets.
%  Reference solution is not computed if refsol = 0. 
%  
%
% Default tol    = 1.0e-4
%         refsol = 1;          
%
%  OMN - 1997

figure(1);clf
figure(2);clf

if nargin < 2
  refsol = 1;
  if nargin < 1
    tol = 1.0e-4;             % Threshold for wavelet coefficients
  end;  
end;

%------------------
% Method parameters
%------------------
p                = 5;         % FD stencil (must be odd)
D                = 8;         % Wavelet genus
wltol            = tol;       % Threshold for wavelet coefs
interpol         = '*spline'; % Interpolation, 'spline' or 'linear'
redothreshold    = 0.02;      % redo when sum(added)/length(added) > redothresh
adaptivetime     = 0;         % False
saveplots        = 0;         % Make eps files of graphs

analytic = 0;
if analytic
  epsilon = 1.0e-7;
else
  epsilon = 1.0e-15;
end;
slowiter = 25;
maxiter = 50;
fs = 16;  %Fontsize

%------------------
% Model parameters
%------------------
n                = 4;                      % 4 if p = 5
%n                = 2,3;                   % 3 if p = 7
Ld               = 64;                     % Domain size             
N                = Ld*2^n;                 % Grid resolution 

if analytic
  beta2            =-2;                   
  gamma            = 2;                    % Non-linearity
  alpha            = 0;                    % Loss
else
  beta2            =-2;                   
  gamma            = 2;                    % Non-linearity
  alpha            = 0;                    % Loss
end;

T                = 2^10;
finaltime        = 0.5;                    %
dt               = 1.0/T;                  % Timestep
time             = 0.0;                    % Simulated time 
plotinterval     = finaltime/8;            % Time between plots
stepsbetweenplot = round(plotinterval/dt);


%----------------------
% Abscissae: x in [-L/2:L/2]
%----------------------
h = Ld/N;   
x = (-Ld/2:h:Ld/2-h)';  % N sample points 

%-----------------
% Setup operators:
%----------------- 
I         = speye(N);        % Sparse NxN identy matrix
[Dx,Dxx]  = fdmat(x,Ld,p);    % Generate FD operators

i  = sqrt(-1);
L  = -i*0.5*beta2*Dxx-alpha/2*I;
A  = I - 0.5*dt*L;   % Crank Nicolson matrices
B  = 2*I - A;        %

[Low, Up] = lu(A);   % LU decomp of A 


%-------------------
% Initial condition:
%-------------------
if analytic
  eta = 1;
  xi  = 1;
  u = eta*sech(eta*x).*exp(i*xi*x);    %initial condition (Corresp to analytic)
else
  u = 2 * sech(x);
end;

%-----------
% Initialize
%-----------
xc   = x;  %Initially, restricted grid is the same as the finest.
uc   = u;
uref = u;  % For reference solution
Dxc  = Dx;
Dxxc = Dxx;
ind  = ones(N,1);
Nc   = length(xc);

clear Dx; 
clear Dxx;

fprintf('N=%d, interval = %6.4f (%d steps), D=%d, p=%d, %s\n',N,plotinterval,stepsbetweenplot,D,p,interpol);

if refsol
%  fprintf('                                         |-------- New Grid ----------|\n')
%  fprintf(' time(interval)   error       #points    #points    #added     #removed\n')
  fprintf('                                |-------- New Grid --------|\n')
  fprintf(' time     error       #points   #points   #added    #removed\n')
else
%  fprintf('                            |-------- New Grid ----------|\n')
%  fprintf(' time(interval)  #points    #points    #added     #removed\n')
  fprintf('                 |-------- New Grid --------|\n')
  fprintf(' time  #points   #points   #added    #removed\n')
end


redoing = 0;  % Reset flag for redoing steps
k       = 0;  % Reset Step counter

%-------------------------
% Compute until final time
%-------------------------

flops_eval      = 0;
flops_solve     = 0;
flops_solve_ref = 0;
cpu_eval        = 0;
cpu_solve       = 0;
cpu_solve_ref   = 0;

while time < finaltime
 if k==0 | k==stepsbetweenplot

   f0 = flops;
   c0 = cputime;

   %-------------------------------
   % Reconstruct u on finest grid x
   %-------------------------------    
   if N==Nc
     u=uc;
   else 
     aug=min(8,Nc);
     xcP = [xc(Nc-aug+1:Nc)-Ld;xc;xc(1:aug)+Ld];
     ucP = [uc(Nc-aug+1:Nc);uc;uc(1:aug)];    
     u = interp1(xcP,ucP,x,interpol);    
   end;

   if refsol
     figure(2); plot(x,abs(uref-u)/max(abs(u))); title('|Error|');
     %figure(2); plot(x,real(uref-u)/max(abs(real(u)))); title('|Error|');
     if saveplots
       eval (['print -depsc2 nls' num2str(time) '_e.eps']);   
     end;  

    err=max(abs(uref-u))/max(abs(u));
   end;
   
    %----------------
    % Define new grid
    %----------------
    %fprintf('Make grid ');pause
    oldind=ind;

    if wltol == 0
      ind=ones(N,1);
    else
      ind     = makegrid(u,D,wltol); % Wavelet based grid evaluator
    end;

    changed = xor(ind,oldind);
    added   = changed & ind;

    %
    % Avoid stagnation
    % Don't remove any if we are redoing the step and still adding!
    %
    if any(added) & redoing  
      ind = added | oldind;
      changed = xor(ind,oldind);
    end
    removed = changed & oldind;

    %------------------------ 
    % Plot grid modifications
    %------------------------

    % Plot restricted uc 
    figure(1);clf;subplot(2,1,2) 
    set(gca,'Fontsize',[fs])    
    hold on
    if any(added)
      xa = x(find(added));
      na = sum(added);
      plot(xa,ones(na,1)/2,'g+',xa,nan*ones(na,1),'r.');
      legend('Pts added', 'Pts removed')
    end;
    if any(removed)
      xr = x(find(removed));
      nr = sum(removed);
      plot(xr,nan*ones(nr,1),'g+',xr,ones(nr,1)/3,'r.');
      legend('Pts added', 'Pts removed')      
    end; 
    axis([-Ld/2 Ld/2 0 1]);    
    title('Grid modifications','Fontsize',fs)
    hold off
    
%    figure(1);subplot(2,1,1);  
%    plot(x(find(added)),ones(sum(added),1)/2,'g+');
%    hold on;
%    plot(x(find(removed)),ones(sum(removed),1)/3,'r.'); 
%    hold off 
%    axis([-Ld/2 Ld/2 0 1]);
%    title('Grid modifications: + = ''added'', - = ''removed'' ')    

    %------------
    % Restrict
    %------------
    
    xc = x(find(ind));
    Nc = length(xc);
    
    %  Quality
    %
    %
    addratio = sum(added)/N;
    remratio = sum(removed)/N; 
    if adaptivetime   
      plotinterval = plotinterval/(1+addratio)*(1+remratio);          % Time between plots
      stepsbetweenplot = round(plotinterval/dt);
    end  
    
    if refsol    
%      fprintf('%5.2f (%5.2f) %12.8f %7d %9d %9d %10d ',time,plotinterval,err,sum(oldind),Nc,sum(added),sum(removed))     
      fprintf('%5.4f %12.8f %7d %9d %9d %10d ',time,err,sum(oldind),Nc,sum(added),sum(removed))     
    else          
%      fprintf('%5.2f (%5.2f) %7d %9d %9d %10d ',time,plotinterval,sum(oldind),Nc,sum(added),sum(removed))             
      fprintf('%5.4f %7d %9d %9d %10d ',time,sum(oldind),Nc,sum(added),sum(removed))             
    end  

    if addratio > redothreshold  % Redo steps on refined grid
      fprintf(' - Redoing step');
      redoing = 1;
      u    = uold;
      time = timeold;
      uref = urefold;       
    else
      fprintf(' ');
      redoing = 0;
      uold    = u;
      timeold = time;
      urefold = uref;       
    end 

    %----------------------------
    % Restrict u to new grid (xc)
    %----------------------------
    uc = u(find(ind));
    points = -0.5*ones(size(xc));  % for plots

    figure(1); subplot(2,1,1); plot(x,abs(u).^2,xc,points,'o'); 
    set(gca,'Fontsize',[fs])    
    legend('Solution','WOFD grid')    
    title(['NLS (t = ', num2str(time), ')'],'Fontsize',fs)
    xlabel('x');
    H=ylabel('|u|^2');
    set(H,'Rotation',0);
    H = get(gca);
    H.YLim(1) = -1;
    set(gca, 'YLim', H.YLim)
    figure(1)
    if saveplots
      eval (['print -depsc2 nls' num2str(time) '.eps']);   
    end;  

            
    %---------------------------------------  
    % Update operators according to new grid
    %---------------------------------------      
    
    [Dxc,Dxxc] = updfdmat(Dxc,Dxxc,x,Ld,p,ind,added,removed);  
    %fprintf('Factorize operators ');pause
    i  = sqrt(-1);
    Ic = speye(Nc);           
    Lc  = -i*0.5*beta2*Dxxc-alpha/2*Ic;
    Ac  = Ic - 0.5*dt*Lc;   % Crank Nicolson matrices
    Bc  = 2*Ic - Ac;        %
    [Lowc, Upc] = lu(Ac);   % LU decomp of Ac 

    fprintf('.');
    fprintf('\n');
    k = 0;
    flops_eval = flops_eval + flops-f0;
    cpu_eval = cpu_eval + cputime-c0;
  end;

  %-------------------------
  % Solve on restricted grid
  %-------------------------
  f0=flops;
  c0=cputime;

  % Iterate nonlinearity 
  error = 1;
  uprev = uc;  
  iter  = 0;

  while error > epsilon
    % Approximate nonlinear part
    umean = (uprev + uc)/2;  
    NL    = dt*i*gamma*abs(umean).^2;  
    b     = Bc*uc + NL.*umean;      
    unew  = Upc\(Lowc\b);           % Solve

    error = max(abs(unew-uprev));
    uprev = unew;

    iter = iter + 1;
    if iter > slowiter
      fprintf('Warning - Slow convergence, iteration # = %d\n',iter);   
      if iter > maxiter
        fprintf('Method does not converge\n');
        return
      end;
    end;         
  end;
  %fprintf('Err = %e, Iterations=%d\n',error,iter); 
  uc = unew;
  flops_solve=flops_solve+flops-f0; 
  cpu_solve=cpu_solve+cputime-c0; 

  %---------------------------
  % Solve on finest grid (ref)
  %---------------------------
  if refsol
    f0ref=flops;
    c0ref=cputime;
  
    % Iterate nonlinearity 
    error = 1;
    uprev = uref;  
    iter  = 0;
  
    while error > epsilon
      % Approximate nonlinear part
      umean = (uprev + uref)/2;  
      NL    = dt*i*gamma*abs(umean).^2;  
      b     = B*uref + NL.*umean;      
      unew  = Up\(Low\b);           % Solve
  
      error = max(abs(unew-uprev));
      uprev = unew;
  
      iter = iter + 1;
      if iter > slowiter
	fprintf('Warning - Slow convergence, iteration # = %d\n',iter);   
	if iter > maxiter
	  fprintf('Method does not converge\n');
	  return
	end;
      end;         
    end;
    %fprintf('Err = %e, Iterations=%d\n',error,iter); 
    uref = unew;
    flops_solve_ref=flops_solve_ref+flops-f0ref; 
    cpu_solve_ref=cpu_solve_ref+cputime-c0ref; 
  end;
  
  %------------------------------
  % Advance time and step counter
  %------------------------------
  time = time + dt;            
  k    = k+1;  
end;

%-------------------------------
% Reconstruct u on finest grid x
%-------------------------------    
if N==Nc
  u=uc;
else 
  aug=min(8,Nc);
  xcP = [xc(Nc-aug+1:Nc)-Ld;xc;xc(1:aug)+Ld];
  ucP = [uc(Nc-aug+1:Nc);uc;uc(1:aug)];    
  u = interp1(xcP,ucP,x,interpol);    
end;

if refsol
  figure(2); plot(x,abs(uref-u)/max(abs(u))); title('|Error|');
  %figure(2); plot(x,real(uref-u)/max(abs(real(u)))); title('|Error|');
  err=max(abs(uref-u))/max(abs(u));
  fprintf('%5.4f %12.8f ',time,err)       
  fprintf('\n');
end

flops_eval      = flops_eval/1.0e6;
flops_solve     = flops_solve/1.0e6;
flops_solve_ref = flops_solve_ref/1.0e6;

fprintf('Eval        = %8.2f MFlop/s, %10d s\n',flops_eval,cpu_eval);
fprintf('Solve       = %8.2f MFlop/s, %10d s\n',flops_solve,cpu_solve);
fprintf('Total(E+S)  = %8.2f MFlop/s, %10d s\n',flops_solve+flops_eval,cpu_solve+cpu_eval)
fprintf('\n');
fprintf('Solve (ref) = %8.2f MFlop/s, %10d s\n',flops_solve_ref,cpu_solve_ref);






%-------------------------------
% Reconstruct u on finest grid x
%-------------------------------    
aug=min(8,Nc);
xcP = [xc(Nc-aug+1:Nc)-Ld;xc;xc(1:aug)+Ld];
ucP = [uc(Nc-aug+1:Nc);uc;uc(1:aug)];    
u = interp1(xcP,ucP,x,interpol);    

if analytic
  t = x;
  z = time;
  % Analytic solution (periodized)
  theta = t - 2*xi*z;
    
  periodized = 0;          % Periodized
  while ~periodized    
    r = (theta < -Ld/2);    %Identify elements left of -L/2
    if sum(r) == 0
      periodized = 1;      % TRUE
    else
      theta = theta + Ld*r; %Periodize	
    end;
  end;
    
  sigma = (eta^2 + xi^2)*z;
  uanal = eta*sech(eta*theta).*exp(i*(xi*theta+sigma));

  error = max(max(abs(u-uanal)))/max(max(abs(u)));
  fprintf('error = %8.2e, log2(error) = %e\n',error,log(error)/log(2));

%  figure(1);
%  plot (t,abs(u),t,abs(uanal),'--');    
else  
%  figure(1)
%  plot (t,abs(u));
%  ylabel('|u|') 
end;  
%if(firstplot), ax=axis; firstplot=0; else axis(ax); end;
%fprintf ('z = %6.3f, Norm(u) = %f, Loss = %f\n',z,norm(u)*sqrt(h),alpha); 

figure(3); plot(x,abs(u).^2,xc,points,'o'); 
set(gca,'Fontsize',[fs])    
title(['NLS WOFD (t = ', num2str(time), ')'],'Fontsize',fs)
xlabel('x');
H=ylabel('|u|^2');    
set(H,'Rotation',0);
legend('Solution','WOFD grid')    
if saveplots
  eval (['print -depsc2 nls' num2str(time) '.eps']);   
end;  


return 
% Zoom solution
N = length(x);
width = N/4;
start = floor(3*width/2);
zoomint = start+1:start+width; 
x1 = x(zoomint);
u1 = u(zoomint);

% Zoom grid points
N = length(xc);
width = N/4;
start = floor(3*width/2);
zoomint = start+1:start+width; 
xc1 = xc(zoomint);
points1 = points(zoomint);
 
figure(4);plot(x1,abs(u1).^2,xc1,points1);

%,xc,points,'o'); 




