function [yfwt,P,err,ratio,abs_thres]=plotw(x,y,D,maxdepth,extension,normalization,scaleconst,threshold,name,xlab,PlotEnergies,space_labels)
%PLOTW  Plot wavelet decomposition of y.
%
%  [yfwt,P]=plotw(x,y,D,maxdepth,extension,normalization,scaling,threshold,name,xlab,PlotEnergies,space_labels);
%
%  Input
%    x: Abscissae vector.
%    y: Function values corresponding to x.
%    D: Wavelet genus. Default: D=8.
%    maxdepth:  
%      Maximal depth (lambda) of decompostion. 
%      Actual depth might be smaller. Default: maxdepth = 2.
%    extension: 
%       Determines how y is extended beyond its boundaries.
%       The possible values are
%       0: Periodic extension (default)
%       1: Extension by reflection
%       2: Polynomial extension
%       3: Smooth wavelet continuation
%    normalization: 
%       Determines how the individual plots are presented.
%       The possible values are
%       0:  Single scale components are plotted on the same scale
%           but with no y-axes.
%       1:  Single scale components are plotted using individual subplots 
%           (default).
%       -1: Like 0, but with each component individually scaled.
%    scaling: 
%       Controls the overall scaling for normalization <= 0.
%       Ignored when normalization = 1
%    threshold:
%       Wavelet coefficients smaller in magnitude than threshold are discarded
%    name: Optional name
%    xlab: Optional xlabel
%    PlotEnergies: Optional (Default: off)
%
% A set of plots are displayed. The bottom graph shows a 
% coarse approximation to y. The graphs above it are levels 
% of succesively finer details of y. 
%
% In case of normalization==-1 a number next to each graph indicates
% an approximate factor by which it should be scaled to obtain its true 
% amlitude.
%
% The topmost graph shows a plot of y.
%
% Ole Moller Nielsen, DTU, October - 96.


blue  = 'b-';  
green = 'k-';

%
% Default values
%
if nargin < 12
  space_labels = 1;
  if nargin < 11
    PlotEnergies = 0;
    if nargin < 10
      xlab = [];
      if nargin < 9
	name = [];
	if nargin < 8
	  threshold = 0;
	  if nargin < 7
	    scaleconst = 1/1.5;
	    if nargin < 6  
	      normalization = 1; % Individual plots
	      if nargin < 5 
		extension = 0; %Periodic extension
		if nargin < 4
		  maxdepth = 2; 
		  if nargin < 3
		    D = 8;
		  end;	
		end;  
	      end;  
	    end;
	  end;
	end;
      end;  
    end;  
  end;
end;

%%%%%%% Parameter check
N = length(x);       % NB x is yhat if called with only one argument!
orgN = N;
if N < 2,
  fprintf(['Error (plotw.m):'...
           'Vector length must greater than 1\n']);
  yfwt = [];
  P = [];
  err = 0;
  ratio = 0;  
  return;	   
end;


if normalization <= 0
  %
  %Scale to unity
  avg  = mean(y);
  y    = y - avg;       %Pull out average
  amp  = max(abs(y));
  y    = y/amp;         %Normalize 
end;

yorg = y; %Save original function to compute error if threshold > 0
[x,y]=extend(x,y,D,maxdepth,extension);

N = length(x);

% Find max power of two
%d = N;
d = orgN;
p = 0;
while rem(d,2) == 0
  p = p + 1;
  d = d/2;
end;  
if p == 0,
  fprintf(['Warning (plotw.m):'...
           'Length must be divisible by a positive power of 2\n']);
  plot(x(1:orgN),y(1:orgN))	  
  Ht=title(name);  
  yfwt = [];
  P = [];
  err = 0;
  ratio = 0;  
  return;	   
end;


lambda = min(p,maxdepth);  % This is the actual depth of the wavelet splitting


NH  = N/2;    
M   = lambda+1;    % Noof rows in hierarchy matrix
x1  = x(1:2:N);    % yhat at finest level has length NH
y   = y(:);        % Force y to be a column

wl   = daubfilt(D);
yhat = fwt(y,wl,lambda); % Do wavelet transform to depth lambda

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Truncate wavelet spectrum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
ratio = 0;     
abs_thres = 0; 
if threshold > 0        
  len = length(yhat);	 
  lenLP = len/2^lambda;          %Length of low pass part  
  lenHP = len - lenLP;           %Length of high pass part  
  yhat_hp = yhat(lenLP+1:end);   %Truncate only high pass part
  [yhat_hp, ratio, abs_thres] = trunc(yhat_hp,threshold);
  yhat(lenLP+1:end) = yhat_hp;
  
  %Update ratio
  ratio = 1 - ((1-ratio)*lenHP + lenLP)/len; 
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct matrix containing all graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P   = zeros(M,N); % row one is finest level, row M-1 is coarsest and 
                  % row M is scaling coefficient at coarsest level (DC). 

last  = N;
for j = 1:M 
  first = last/2;
  yhat1 = zeros(N,1);  % Vector containing coefficients at level j
  if j < M
    yhat1(first+1:last) = yhat(first+1:last);
  else
    yhat1(1:last) = yhat(1:last);  % DC-term     
  end; 
  P(j,:) = ifwt(yhat1, wl, lambda)';  % Scaling coefficients of projection 
  
  last = first;
end;  		   
		   
if extension > 0
  % Remove reflected parts 
  N = orgN;
  P = P(:,1:N);
  x = x(1:N);
  y = y(1:N);
  
  yfwt = zeros(size(y));
  
  last0 = length(yfwt); %Index into output wavelet spectrum
  last1 = length(yhat); %Index into extended wavelet spectrum  
  for j=1:M
    first0 = last0/2;
    len = last0 - first0;
    first1 = last1/2;    
    if j < M    
      yfwt(first0+1:last0) = yhat(first1+1:first1+len);
    else  
      yfwt(1:last0) = yhat(1:last0);    
    end;  
    last0 = first0;
    last1 = first1;    
  end;
else
  yfwt = yhat;  
end;

sumP = sum(P);	
		 

%%%%%%%%%%%% Plot

if normalization ==  -1
  graphname = ['Wavelet decomposition (normalized), D=' int2str(D)];
elseif normalization == 1
  graphname = ['Wavelet decomposition (individual plots), D=' int2str(D)];
else
  graphname = ['Wavelet decomposition, D=' int2str(D)];
end; 
if extension == 0
%  graphname = [graphname ' - extended periodically'];
elseif extension == 1
  graphname = [graphname ' - extended by reflection'];  
elseif extension == 2  
  graphname = [graphname ' - extended polynomially'];  
elseif extension == 3  
  graphname = [graphname ' - extended by SWC'];  
end; 

if ~isempty(name)
  if ischar(name)
    graphname = char(name, graphname);
    %%%%%%%%%%%% centre name and graphname for title function:
    difference = length(deblank(graphname(1,:))) - length(deblank(graphname(2,:)));
    addblanks = abs(floor(difference/2));
    if difference < 0, 
      graphname = char([repmat(' ',1,addblanks) deblank(graphname(1,:))],graphname(2,:));
    else 
      graphname = char(graphname(1,:), [repmat(' ',1,addblanks) deblank(graphname(2,:))]);
    end
  else
    graphname = '';
  end;
else
  graphname = '';  
end;
    


%%%%%%%%%%%%%%%%%%%%% PLOT %%%%%%%%%%%%%%%%%%%%%
if normalization <= 0
  clf
  hold on

  scales = zeros(M,1);  
  offset = 1;  
  
  if normalization == -1
    for j = 1:M
      scales(j) = max(abs(P(j,:)));   % Maximal amplitude
    end;
  else
    scales = ones(M,1);                     % No scaling  
  end;    
  
  scales = scales/scaleconst;
  
  for j = 1:M
    Psc(j,:) = P(j,:)/scales(j);  % Normalize each level
  end;
  
  Pud    = flipud(Psc);
  scales = flipud(scales);

  for j=1:M                    %Plot single scale components bottom up
    ypos = (j-1)*offset;
    plot(x,Pud(j,:) + ypos,blue);    	    

    level = j-2 + p-lambda;        
    
    if space_labels           
      if level < p-lambda, str = ['V_{' int2str(level+1) '}'];
      else                 str = ['W_{' int2str(level) '}'];
      end;
    else
      if level < p-lambda, str = ['s_{' int2str(j-1) '}'];
      else                 str = ['d_{' int2str(j) '}'];
      end;
    end;      

    factor = round(scales(j));
    if normalization == -1 
      levstr = [str, ' *' int2str(factor)];
    else
      levstr = str;
    end

    ax    = axis;  
    xpos = ax(2) + 0.02*(ax(2)-ax(1)); % Place text immediately to the right of graph.    
    text(xpos,ypos,[levstr])
  end;
  ypos = M*offset;
  
  scal = max(scales);
%  plot(x,sum(P)/scal + ypos,green);  
  plot(x,sumP/scal + ypos,green);  
  
  if space_labels
    str = ['V_{' int2str(level+1) '}'];
  else
    str = ['s'];  
    %str = [int2str(lambda) 's'];  
  end;
  
  factor = ceil(scal);
  if normalization == -1 & factor > 1
    levstr = [str, ' *' int2str(factor)];
  else
    levstr = str;
  end
  
  text(xpos,ypos,[levstr])  
  Ht=title(graphname);  
  
  set(gca,'Ytick',[])
  set(gca,'Yticklabel',['-'])  % Could be used to mark zero for each graph   
  
elseif normalization == 1
  clf
    
  %Plot single scale components top down with energies on the right
   
  M1 = M+1;
  %
  % Create figure
  %
  for j=1:M1
    if PlotEnergies
      Extendfactor = 1.9;    
      Reducefactor = 0.35;          
      Displacement = 0.25; 
      
      H2(j) = subplot(M1,2,j*2);    
      POS = get(H2(j),'Position');
      POS(1) = POS(1)+Displacement;      
      POS(3) = POS(3)*Reducefactor;
      set(H2(j),'Position',POS);
      
    
      H1(j) = subplot(M1,2,(j-1)*2+1);
      POS = get(H1(j),'Position');
      OldEdge = POS(1)+POS(3);
      POS(3) = POS(3)*Extendfactor;
      set(H1(j),'Position',POS);
      
    else
      H1(j) = subplot(M1,1,j);      
    end;  
  end;
  
  
  % Plot function at finest level
  subplot(H1(1));  
%  plot(x,sum(P),green);  
  plot(x,sumP,green);    
  ax = axis;  %Save axis
  xmin = ax(1);
  xmax = ax(2);
  
  set(gca,'Xtick',[]);  
  Ht=title(graphname);  
  
  level = M-2 + p-lambda;          
  if space_labels
    levstr = ['V_{' int2str(level+1) '}'];
  else
    levstr = ['s_{0}'];  
%    levstr = [int2str(lambda) 's'];  
  end;  
    
  ax    = axis;  
  xpos = ax(2) + 0.02*(ax(2)-ax(1)); % Place text immediately to the right of graph.      
  ypos  = ax(3);    
  text(xpos,ypos,[levstr])
  
  % Plot decompositions
  for j=1:M       
    subplot(H1(j+1));
    plot(x,P(j,:),blue);         
    ax = axis;  %Force same axis as finest level
    ax(1) = xmin;
    ax(2) = xmax;
    axis(ax);
    
    if j < M;
      set(gca,'Xtick',[]);
%    else
%      deltax = x(end)-x(end-1);
%      set(gca,'Xtick',[x(1) x(end)+deltax]);        
    end;
    
    level = (M-j+1)-2 + p-lambda;    
    if space_labels    
      if level < p-lambda, levstr = ['V_{' int2str(level+1) '}'];
      else                 levstr = ['W_{' int2str(level) '}'];
      end;
    else
      if level < p-lambda, levstr = ['s_{' int2str(j-1) '}'];
      else                 levstr = ['d_{' int2str(j) '}'];
      end;
    end;  
    ax    = axis;  
    %xpos  = ax(2)*1.02;  %OLD
    xpos = ax(2) + 0.02*(ax(2)-ax(1)); % Place text immediately to the right of graph.
    ypos  = ax(3);    
    text(xpos,ypos,[levstr])
  end;
  
  if PlotEnergies
    eplot(yfwt,H2,lambda)
  end;
end;

%
% Left justify title if it consists of more than one line
% - not needed with centred graphname variable.
%
% if ~isempty(name)
%  set(Ht,'HorizontalAlignment','left');
%  Tpos = get(Ht,'Position');
%  Tpos(1)=0;
%  set(Ht,'Position',Tpos);
%end;

if ~isempty(xlab)
  subplot(H1(end));
  xlabel(xlab);
end;


%Compute relative error
err = norm(y - sumP')/norm(y);



function eplot(yfwt,H,lambda)
%
%
%

Res = 100;
x = (0:Res)'/Res;
y = 0.5*ones(size(x));

E = scaleenergy(yfwt,lambda);
E = [1, E']';

%% Uncomment if log scale is wanted
%%E = log(E);
%%[E,Emin,Scale] = normalise(E);

N = length(E);
for n = 1:N
  subplot(H(n));
  plot(x,y,'.','MarkerSize',2);
  hold on
  plot(E(n),0.5,'*')
  hold off
  axis([0 1 0 1])
  
  if n == 1
    title('Energies');    
  end;  
  if n < N
    set(H(n),'XTick',[])
  else
    set(H(n),'XTick',[0 1])  
    %set(H(n),'XTickLabel',[exp(Emin) 1])      
  end;  
  set(H(n),'YTick',[])  
end;  









