function plotws(x,f,D,reflect,maxdepth)
%PLOTWS  Plots periodic wavelet expansion of f separated according to scale.
%
%
%  plotws(x,f,D,e,depth)
%
%  Input
%    x: Abscissae vector - must be a power of two.
%    f: Function values corresponding to x.
%    D: Wavelet genus. Default: D=4.
%    e: If e~=0 then f is assumed to extend outside its domain by reflection.
%               Otherwise it is extended periodically. Default: e=0.
%
%
% A set of graphs are displayed as subplots with individual axes. 
% The bottom graph shows the the DC terms, the
% graphs above are level of succesively finer details of f as 
% decomposed by the wavelet transform. 
%
% The topmost graph shows the function f.

%
% Ole Moller Nielsen, ANU, 1999.


blue  = 'b-';  
green = 'k-';

if nargin < 5  
  maxdepth = 4;
  if nargin < 4 
    reflect = 0;
    if nargin < 3
      D = 8;
    end;	
  end;  
end;


if reflect
  %%%%%%% Mirror
  x=x(:);f=f(:); %Force vecors to be columns
  x = [x;x+1];
  f = [f;flipud(f)];
end;

%%%%%%% Parameter check
N   = length(x);       % NB x is fhat if called with only one argument!
if N < 2,
  fprintf(['Error (plotws.m):'...
           'Vector length must greater than 1\n']);
  return;	   
end;

% Find max power of two
d = N;
p = 0;
while rem(d,2) == 0
  p = p + 1;
  d = d/2;
end;  
if p == 0,
  fprintf(['Error (plotws.m):'...
           'Length must be divisible by a positive power of 2\n']);
  return;	   
end;

if reflect
  p = p-1;  %Because vector was artificially doubled in case of extension by reflection          
end;  

lambda = min(p,maxdepth);  % This is the actual depth of the wavelet splitting

NH  = N/2;    
M   = lambda+1;             % Noof rows in hierarchy matrix

x1  = x(1:2:N);        % fhat at finest level has length NH

[rows,cols] = size(f);
if (N == cols) f = f'; end;   %fhat must be a column vector;

wl   = daubfilt(D);
c  = f;   % Shortcut  


fhat = fwt(c,wl,lambda); % Do wavelet transform to depth lambda


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct matrix containing all graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P   = zeros(M,N); % row one is finest level, row M-1 is coarsest and 
                  % row M is scaling coefficient at coarsest level (DC). 

last  = N;
for j = 1:M 
  first = last/2;
  fhat1 = zeros(N,1);  % Vector containing coefficients at level j
  if j < M
    fhat1(first+1:last) = fhat(first+1:last);
  else
    fhat1(1:last) = fhat(1:last);  % DC-term     
  end; 
  c1 = ifwt(fhat1, wl, lambda);  % Scaling coefficients of projection 
  P(j,:) = c1';
  
  last = first;
end;  		   
		   
if reflect		   
  % Remove reflected parts
  N = N/2;
  P = P(:,1:N);
  x = x(1:N);
  f = f(1:N);
end;
		   
%%%%%%%%%%%% Plot


graphname = ['Wavelet decomposition (individual plots), D=' int2str(D)];

if reflect
  graphname = [graphname ' - extended by reflection'];
else
  graphname = [graphname ' - extended periodically'];
end; 
    


%%%%%%%%%%%%%%%%%%%%% PLOT %%%%%%%%%%%%%%%%%%%%%
  clf
    
  %Plot single scale components top down  
    
  M1 = M+1;
  % Plot function at finest level
  subplot(M1,1,1);  
  plot(x,f,green);  title(graphname);  
  level = M-2 + p-lambda;          
  levstr = ['V_{' int2str(level+1) '}'];
  ax    = axis;  
  xpos  = ax(2)*1.02;  % Place text immediately to the right of graph.
  ypos  = ax(3);    
  text(xpos,ypos,[levstr])
  
  % Plot decompositions
  for j=1:M       
    subplot(M1,1,j+1);
    plot(x,P(j,:),blue);         
    
    level = (M-j+1)-2 + p-lambda;        
    if level < p-lambda, levstr = ['V_{' int2str(level+1) '}'];
    else                 levstr = ['W_{' int2str(level) '}'];
    end;
    ax    = axis;  
    xpos  = ax(2)*1.02;  % Place text immediately to the right of graph.
    ypos  = ax(3);    
    text(xpos,ypos,[levstr])
  end;

%  subplot(M+1,1,M+1);
%  plot(x,sum(P)/scal + ypos,blue,x,f/scal+ypos,green);  
%  str = ['V_{' int2str(level+1) '}'];
%  
%  factor = ceil(scal);
%  if normalize & factor > 1
%    levstr = [str, ' *' int2str(factor)];
%  else
%    levstr = str;
%  end
  
%  text(xpos,ypos,[levstr])  



%set(gca,'Ytick',[])
%set(gca,'Yticklabel',['-'])  % Could be used to mark zero for each graph 














