function [X,NH] = pwt(X, N, lp, hp)
%PWT Partial (Single stage) Wavelet Transform.
%
%  [X, NH] = pwt (X, N, lp, hp);
%
% Perform one step of the wavelet transform 
% on the first N components of each column of X.
% the vectors lp and hp contain the wavelet filter coefficients.
%
% See also: DAUB, FWT, IFWT, and IPWT
%
% Ole Moller Nielsen - 16 April 1997.


D   = length (lp);
if D ~= length(hp);
  fprintf('ERROR (pwt): Filters should be of same length. Pad shortest one with zeros.');
end;

T   = zeros (size(X));  % Workspace

NH  = N/2;
NH1 = NH+1;

for k=1:D            % Loop over wavelet coefficients
  k2 = k-2;       
  if k2 < 1          % Determine which part of W must be computed with mod()
    crossover = 0;                   
  elseif k2 < N
    crossover = ceil (k2/2);  
                % Guarantees that k+2i-3 < N for i = 1,NH-crossover
  else  
    crossover = NH;
  end;   

  % --------------- Linear phase -----------------
  L            = NH-crossover;
  if L > 0
    NHL          = NH+L;
    Xsub         = X(k:2:2*L-2+k,:);      %Every other element of linear part
    T(1:L,:)     = T(1:L,:)     + lp(k)*Xsub;
    T(NH1:NHL,:) = T(NH1:NHL,:) + hp(k)*Xsub;
  end;
  % --------- Modulus phase  - only a fraction of the total time is spent here 
  for i=L+1:NH  
    r      = rem(k+2*i-3,N) + 1;    % Row number in X
    j      = i+NH;                  % Row number in second half of result
    T(i,:) = T(i,:) + lp(k)*X(r,:);
    T(j,:) = T(j,:) + hp(k)*X(r,:);
  end;
  clear Xsub;
end;

X(1:N,:) = T(1:N,:);         % Copy wavelet transform at level N back into X
clear T;


































