function [Y, ratio, thres]=trunc(X, th, mode)
%TRUNC
%
%  [Y, ratio, thres]=trunc(X, th, mode) 
%
%  Removes elements from X which are smaller than th*W(X)
% 
%  W(X) depends on the value of mode (default = inf):
%
%    mode    W(x)
%    --------------
%    inf     norm(x,inf)
%    2       norm(x,2)
%    1       norm(x,1)
%    0       1           (absolut tolerance)
%
%  th should belong to [0,1] unless mode == 0. Default: th=eps
%
%  Return values:
%    Y:     the matrix with small elements replaced by zeros
%    ratio: (in [0,1]) the fraction of elements which were set to zero.
%    thres: the absolute threshold that was used

% Ole Moller Nielsen, ANU, October - 95.
%      -  modified at NYU June - 96.

if nargin < 3
  mode = inf;
  if nargin < 2
    th = eps;
  end;  
end;

if length(X)==0
  error('Null vector given as argument');
end;  

if mode == 0
  W = 1;
else
  if mode == inf
    W = full(max(max(abs(X))));  
  else
    W = norm(full(X),mode);
  end;
end;  
   
thres = W*th;
if isnan(thres)
  S=sprintf('Threshold is NaN, W(X) = %4.2e\n', W);
  warning(S)
end

mask         = (abs(X)>thres);
Y            = X.*mask;
[xdim, ydim] = size(mask);
total        = xdim*ydim;
nelem        = total - sum(sum(mask));   % Elements below threshold
ratio        = nelem/total;






