function wavemenu(sw)
%WAVEMENU Graphical front end to wavelet package.
%
% WAVEMENU provides a menu for performing 
%    1D wavelet analysis using the wavelet package.
%
% Data to be analyzed can be provided either as global 
% vectors xglob and yglob or in the form of an
% input file consisting of two columns of numbers.
%
% Calling wavemenu from within another Matlab function
% using an N by 2 array data has the following form:
%
%   global xglob yglob
%   xglob = data(:,1);
%   yglob = data(:,2);
%   wavemenu;
%
% A number of example files are provided with the toolbox.
%
%
% For help on the underlying toolbox type 'help contents'.
%
% OMN, ANU - 1998

global Dglob dglob scglob normglob exttypeglob
global xglob yglob 
global nameglob xlabglob


if nargin < 1, 
  sw = 'main'; 
end;  

%Constants
Scalemax = 2;    % Maximal scaling (2)
Scalemin = 1/10; % Minimal scaling (1/10)

tpowmax =  0; % Maximal threshold (10^tpowmax)
tpowmin = -3; % Minimal threshold (10^tpowmin)


%----------------------------------------------------------
% Initialize GUI
%---------------------------------------------------------
if strcmp(sw,'main') %Define the controls
  H1 = 0;
  H2 = 0;
  H3 = 0;  
  H4 = 0;
  OK = 0;
  
  %Default values
  
  x    = xglob(:);    %Data (OK is empty)
  y    = yglob(:);    %
  xlab = xlabglob;    %
  name = nameglob;    %
  
  if isempty(name),
    name = 'Data';
  end;	 
  
  if isempty(dglob)
    d = 2;     % Transform depth
  else
    d = dglob;  
  end;
  
  if isempty(Dglob)  
    D = 8;     % Wavelet genus
  else
    D = Dglob;  
  end;  

  if isempty(scglob)
    sc = 0.5;   % Scaling
  else
    if scglob > Scalemax 
      sc = Scalemax; 
    elseif scglob < Scalemin  
      sc = Scalemin;
    else  
      sc = scglob;
    end;  
  end;    

  tpow = tpowmin;
%  if isempty(scglob)
%    sc = 0.5;   % Scaling
%  else
%    if scglob > Scalemax 
%      sc = Scalemax; 
%    elseif scglob < Scalemin  
%      sc = Scalemin;
%    else  
%      sc = scglob;
%    end;  
%  end;    
  
  
  if isempty(normglob)  
    norm = 1;           % Normalize each scale separately 
  else
    norm = normglob;
  end;
  
  if isempty(exttypeglob)    
    exttype  = 0;     % Extend periodically
  else
    exttype = exttypeglob;
  end;    
    
  
  % Build main form
  H1 = figure;clf;
  H1dat = get(H1);  
  BG    = H1dat.Color; 
  % Initialize H1 window
  %
  pos1 = H1dat.Position;
  pos1(1) = pos1(1)/1.8;  %Move to the left
  set(H1,'Position',pos1);
  set(H1,'MenuBar','none');
  set(H1,'Name','Wavelet menu 1D');
  
  
% Constants for positioning graphical objects    
  xpos0   = 20; 
  xpos1   = 180;    
  xpos2   = 300;
  xpos3   = 382;  
  ypos0   = 380;
  ypos1   = 380;  
%  xwid0   = 90; xwid0=90 is nice for Linux   
  xwid0   = 112;    
  xwid1   = 160;        
  ywid0   = 20;
  xoff    = 60;     
  yoff    = 60;   

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Buttons for choosing parameters
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  %FRAME  
  h=8;
  HcFrame = uicontrol(H1,'Style','frame',...			
			 'Position',...
		[xpos0-5 ypos0-h*ywid0-5-2 xwid0+xwid1+10 (h+1)*ywid0+10]);
  
  %HEADER		             
  HcTitle = uicontrol(H1,'Style','text',...			 
			 'Position',[xpos0 ypos0 xwid0+xwid1 ywid0],...
			 'String','Parameters',...
			 'HorizontalAlignment','center');	
  
  %SEPARATOR		       
  HcSep = uicontrol(H1,'Style','frame',...
		       'Position',...
		       [xpos0-5 ypos0+3 xwid0+xwid1+10 2]);
  
  %-------------------------------------------------------------------	     
  %NORMALIZATION		     				
  ypos0 = ypos0 - ywid0;
  HcNormLabel = uicontrol(H1,'Style','text',...			 
			'Position',[xpos0 ypos0-3 xwid1 ywid0],...
			  'String','Normalize each scale separately',...
			  'HorizontalAlignment','left');		
			  
  HcNorm = uicontrol(H1,'Style','check',...
			'Position',[xpos1 ypos0 xwid0 ywid0],...
			'Callback','wavemenu(''wd'');');
  set(HcNorm,'Value',norm)
  val = get(HcNorm,'Value');
  if val, set(HcNorm,'String','yes','HorizontalAlignment','left');
  else set(HcNorm,'String','no','HorizontalAlignment','left');
  end;
  
  %-------------------------------------------------------------------
  %EXTENSION			 
  ypos0 = ypos0 - ywid0;			 
  HcExtLabel = uicontrol(H1,'Style','text',...			 
			    'Position',[xpos0 ypos0-3 xwid1 ywid0],...
			    'String','Signal extended by',...
			    'HorizontalAlignment','left');
  
  %HcExt = uicontrol(H1,'Style','check',...
  %                     'Position',[xpos1 ypos0 xwid0 ywid0],...
  %                     'Callback','wavemenu(''wd'');');
  HcExt = uicontrol(H1,'Style','popupmenu',...
		       'Position',[xpos1 ypos0 xwid0 ywid0],...
		       'String',...
		       ['Periodization|Reflection|Polynomial|SWC'],...
		       'Value',exttype+1,...
		       'Callback','wavemenu(''wd'');');
			   
  %-------------------------------------------------------------------
  %WAVELET GENUS
  ypos0 = ypos0 - ywid0;			 			 
  HcDLabel = uicontrol(H1,'Style','text',...			 
			  'Position',[xpos0 ypos0-3 xwid1 ywid0],...
			  'String','Wavelet genus',...
			  'HorizontalAlignment','left');
  
  HcD = uicontrol(H1,'Style','popupmenu',...
		 'Position',[xpos1 ypos0 xwid0 ywid0],...
		 'String',...
		 ['D=2  |D=4  |D=6  |D=8  |D=10|D=12|D=14|D=16|D=18|',...
		 'D=20|D=22|D=24|D=26|D=28|D=30|D=32|D=34|D=36|D=38|D=40'],...
		 'Value',D/2,...
		 'Callback','wavemenu(''wd'');');
  
  %-------------------------------------------------------------------
  %TRANSFORM DEPTH
  ypos0 = ypos0 - ywid0;			 			 
  HcDepthLabel = uicontrol(H1,'Style','text',...			 
			  'Position',[xpos0 ypos0-3 xwid1 ywid0],...
			  'String','Transform depth',...
			  'HorizontalAlignment','left');
  
  HcDepth = uicontrol(H1,'Style','popupmenu',...
	    'Position',[xpos1 ypos0 xwid0 ywid0],...
	    'String',...
	    ['d=1  |d=2  |d=3  |d=4  |d=5  |d=6  |d=7  |d=8  |d=9  '],...
	    'Value',d,...
	    'Callback','wavemenu(''wd'');');
  
  
  %-------------------------------------------------------------------
  %SCALING
  ypos0 = ypos0 - ywid0;			 			 
  HcScaleLabel = uicontrol(H1,'Style','text',...			 
			  'Position',[xpos0 ypos0-3 xwid1 ywid0],...
			  'String','Scaling',...
			  'HorizontalAlignment','left');
  
  HcScale = uicontrol(H1,'Style','slider',...
		     'Position',[xpos1 ypos0 xwid0 ywid0],...
		     'Min',Scalemin,'Max',Scalemax,'Value',sc,...
		     'SliderStep',...
		     [0.1/(Scalemax-Scalemin) 0.5/(Scalemax-Scalemin)],...   
		     'Callback','wavemenu(''wd'');');
  
  HcScaleVal = uicontrol(H1,'Style','text',...			 
			'Position',[xpos0+70 ypos0-3 50 ywid0],...
			'String',num2str(sc),...
			'HorizontalAlignment','right');
  
  %-------------------------------------------------------------------
  %DENOISING
  ypos0 = ypos0 - ywid0;			 			 
  
  HcEpsLabel = uicontrol(H1,'Style','text',...			 
		  'Position',[xpos0 ypos0-3 xwid1 ywid0],...
		  'String','Rel threshold',...
		  'HorizontalAlignment','left');
  
  HcEps = uicontrol(H1,'Style','slider',...
		     'Position',[xpos1 ypos0 xwid0 ywid0],...
		     'Min',tpowmin,'Max',tpowmax,'Value',tpow,...
		     'SliderStep',...
		     [0.01/(tpowmax-tpowmin) 0.1/(tpowmax-tpowmin)],...   
		     'Callback','wavemenu(''wd'');');
  
  HcEpsVal = uicontrol(H1,'Style','text',...			 
		        'Position',[xpos0+70 ypos0-3 50 ywid0],...
			'String', num2str(0),...
			'HorizontalAlignment','right');

			
  %SEPARATOR		       
  HcSep = uicontrol(H1,'Style','frame',...
		       'Position',...
		       [xpos0-5 ypos0-5 xwid0+xwid1+10 2]);
 
  %-------------------------------------------------------------------
  %REPORT Absolute threshold used
  ypos0 = ypos0 - ywid0;			 			 
  yoff = 2;

  HcThrLabel = uicontrol(H1,'Style','text',...			 
		  'Position',[xpos0 ypos0-3-yoff xwid1 ywid0],...
		  'String','Abs threshold',...
		  'HorizontalAlignment','left');
  
  HcThrVal = uicontrol(H1,'Style','text',...			 
		        'Position',[xpos0+70 ypos0-3-yoff 50 ywid0],...
			'String', num2str(0),...
			'HorizontalAlignment','right');
  
  %-------------------------------------------------------------------
  %REPORT COMPRESSION
  %ypos0 = ypos0 - ywid0;			 			 
  
  HcCompLabel = uicontrol(H1,'Style','text',...			 
		  'Position',[xpos1 ypos0-3-yoff 80 ywid0],...
		  'String','Comp ratio',...
		  'HorizontalAlignment','left');
  
  HcCompVal = uicontrol(H1,'Style','text',...			 
		        'Position',[xpos1+60 ypos0-3-yoff 50 ywid0],...
			'String', num2str(0),...
			'HorizontalAlignment','right');
  
  

  %-------------------------------------------------------------------
  %REPORT ERROR
  ypos0 = ypos0 - ywid0;			 			 
  yoff = 2;

  HcErrLabel = uicontrol(H1,'Style','text',...			 
		  'Position',[xpos0 ypos0-3-yoff xwid1 ywid0],...
		  'String','Error',...
		  'HorizontalAlignment','left');
  
  HcErrVal = uicontrol(H1,'Style','text',...			 
		        'Position',[xpos0+70 ypos0-3-yoff 50 ywid0],...
			'String', num2str(0),...
			'HorizontalAlignment','right');
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Command buttons
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  %FRAME 
  h=6;   
  HcFrame2 = uicontrol(H1,'Style','frame',...			
			 'Position',...
			 [xpos2-5 ypos1-h*ywid0-5 xwid0+xwid1+10 (h+1)*ywid0+10]);
  
  %HEADER
  HcTitle2 = uicontrol(H1,'Style','text',...			 
			 'Position',[xpos2 ypos1 xwid0+xwid1 ywid0],...
			 'String','Commands',...
			 'HorizontalAlignment','center');
  %SEPARATOR
  HcSep2 = uicontrol(H1,'Style','frame',...
			'Position',...
			[xpos2-5 ypos1+3 xwid0+xwid1+10 2]);
  
  %-------------------------------------------------------------------
  %LOAD DATA FROM FILE
  ypos1 = ypos1 - ywid0;			 			 	       
  HcGetfile = uicontrol(H1,'Style','push',...
			   'String','Load data',...
			   'Position',[xpos2 ypos1 xwid0 ywid0],...
			   'Callback','wavemenu(''getfn'');wavemenu(''wd'');');
  
  %figure(H1);subplot(2,1,2);plot(x,y);title('No file chosen');
			   
  %-------------------------------------------------------------------
  %DISPLAY WAVELET SPECTRUM
  ypos1 = ypos1 - ywid0;			 			 
  HcView = uicontrol(H1,'Style','push',...
		       'String','Wavelet spectrum',... 
		       'Position',[xpos2 ypos1 xwid0 ywid0],...
		       'Enable','off',... 
		       'Callback','wavemenu(''view'');');
			   
  %-------------------------------------------------------------------
  %DISPLAY EXTENSION
  ypos1 = ypos1 - ywid0;			 			 
  HcViewExt = uicontrol(H1,'Style','push',...
		       'String','View extension',... 
		       'Position',[xpos2 ypos1 xwid0 ywid0],...
		       'Enable','off',... 
		       'Callback','wavemenu(''extension'');');
			   
  %-------------------------------------------------------------------
  %REFRESH
  ypos1 = ypos1 - ywid0;			 			 
  HcWDA = uicontrol(H1,'Style','push',...
		       'String','Refresh',... 
		       'Position',[xpos2 ypos1 xwid0 ywid0],...
		       'Enable','off',... 
		       'Callback','wavemenu(''wd'');');
	
  %-------------------------------------------------------------------
  %HELP
  ypos1 = ypos1 - ywid0;			 			 
  HcHelp = uicontrol(H1,'Style','push',...
			'String','Help',...
			'Position',[xpos2 ypos1 xwid0 ywid0],...		 
			'Callback','help wavemenu');
	
	
  %-------------------------------------------------------------------
  %EXIT
  ypos1 = ypos1 - ywid0;			 			 
  HcExit = uicontrol(H1,'Style','push',...
			'String','Exit',...
			'Position',[xpos2 ypos1 xwid0 ywid0],...		 
			'Callback','wavemenu(''cleanup'');');
			      
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Initialization and data check			 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
  if isempty(x)
    OK   = 0;
    rows = 0;
    x    = 0;
    y    = 0;
  else
    % Check length and divisibility by a power of 2
    rows = length(x);
    if rows < 2
      warndlg('Vector length is less than 2');  
      OK = 0;
    else
      OK = 1;  
    end;
  end;  
  
  yorg = y;           %

  Figures = [H1 H2 H3 H4];
  Handles  = ... 
    [HcWDA HcView HcNorm HcExt HcD HcDepth HcScale HcScaleVal HcEps HcEpsVal HcViewExt HcErrVal HcCompVal HcThrVal]; 
  Numvars  = [rows norm exttype D d OK];
  Textvars = {name, xlab};
  Data     = [x y yorg];
  Globals = {Figures, Handles, Numvars, Textvars, Data};   
  set(H1,'UserData',Globals);    
  
  
  if OK
    maxd = 0;
    M = rows;
    while rem(M,2) == 0
      maxd = maxd + 1;
      M = M/2;
    end;  
  
    if d > maxd
      warndlg(...
	 ['The maximal transform depth for this signal is d=' num2str(maxd)]);
	 d = maxd;
	 set(HcDepth,'Value',d);         
    end;
  
  
  
    % Try to determine type of initial extension (periodic or reflection)
    % based values at the end points.
    if isempty(exttypeglob)
      delta = abs(y(rows)-y(1))/max(abs(y));
      if delta > 0.01
        exttype = 1;   %Extend by reflection
      else
        exttype = 0;   %Extend by periodization
      end;
    end;  
  
    set(HcExt,'Value',exttype+1);  
    set(HcWDA,'Enable','on');
    set(HcView,'Enable','on');  
  
%    eval([mfilename '(''fwt'')']); %Compute wavelet spectrum and determine  
%	 			   %if normalization is reasonable
    wavemenu('wd');
  end;			    

  
else	
			    			    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % CALLBACK FUNCTIONS
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  Globals = get(gcf,'UserData');

  Figures  = Globals{1};
  Handles  = Globals{2};  
  Numvars  = Globals{3};
  Textvars = Globals{4};  
  Data     = Globals{5};  

  H1 	     = Figures(1);
  H2 	     = Figures(2);
  H3 	     = Figures(3);
  H4 	     = Figures(4);  
    
  HcWDA      = Handles(1);
  HcView     = Handles(2);
  HcNorm     = Handles(3);
  HcExt      = Handles(4);
  HcD        = Handles(5);
  HcDepth    = Handles(6);
  HcScale    = Handles(7);
  HcScaleVal = Handles(8);
  HcEps      = Handles(9);
  HcEpsVal   = Handles(10);
  HcViewExt  = Handles(11);
  HcErrVal   = Handles(12);
  HcCompVal  = Handles(13);      
  HcThrVal   = Handles(14);        

  rows 	     = Numvars(1);
  norm 	     = Numvars(2);
  exttype    = Numvars(3);
  D 	     = Numvars(4);
  d 	     = Numvars(5);
  OK	     = Numvars(6);
  
  name       = Textvars{1};
  xlab       = Textvars{2};
  
  x          = Data(:,1);
  y          = Data(:,2);
  yorg       = Data(:,3);  
  
  figure(H1); set(H1,'pointer','watch');   
    
  %----------------------------------------------------------
  % Get data file
  %---------------------------------------------------------
  if strcmp(sw,'getfn')
  
  [FN, PN] = uigetfile('*.dat',...
  'Enter filename with signal to be analyzed',450,30);
  
  
  if FN
 
  %  eval(['S=load('' ' FN ' '');']) % Works only with matlab for Linux
  
    eval(['load ' PN FN]);  
    k = length(FN);
    while FN(k) ~= '.', k = k-1; end; %Find length of file ext  
    eval(['S = ', FN(1:k-1), ';']);
  
    [rows cols] = size(S);  
    if H3, figure(H3); delete(H3); H3=0; end; %Remove window with raw wavelet coefs
    if H4, figure(H4); delete(H4); H4=0; end; %Remove window with extension    
    OK = 1;
    
    % Check divisibility by a power of 2
    M = rows;
    maxd = 0;
    while rem(M,2) == 0
      maxd = maxd + 1;
      M = M/2;
    end;  
  
    if d > maxd
      warndlg(...
	 ['The maximal transform depth for this signal is d=' num2str(maxd)]);
	 d = maxd;
	 set(HcDepth,'Value',d);         
    end;
	
    % Assign values to x and y vectors
    if cols == 2   
      x = S(:,1);           %first column is x          
      y = S(:,2);           %second column is y          
    elseif cols == 1
      y = S(:,1);           %single column is y             
      x = (0:rows-1)'/rows; %default abscissae
    else
      errordlg('File must consist of one or two columns of numbers.')  
      OK = 0;
    end;        
  else
  %  OK = 0;  
  end;  
  
  if OK
    % Try to determine type of initial extension (periodic or reflection)
    % based values at the end points.
    delta = abs(y(rows)-y(1))/max(abs(y));
    if delta > 0.01
      exttype = 1;   %Extend by reflection
    else
      exttype = 0;   %Extend by periodization
    end;
    
    set(HcExt,'Value',exttype+1);  
    set(HcWDA,'Enable','on');
    set(HcView,'Enable','on');  
    name = FN;
    yorg = y;
    
    
%    wavemenu('fwt'); %Compute wavelet spectrum and determine  
%		     %if normalization is reasonable
  end
  
  %----------------------------------------------------------
  % Do wavelet decomposition
  %----------------------------------------------------------
  elseif strcmp(sw,'wd')
    % Get chosen values from button objects and update their
    % string attributes.
    
    D       = 2*get(HcD,'Value');    
    d       = get(HcDepth,'Value'); 
  
    sc      = round(10*get(HcScale,'Value'))/10;
    set(HcScaleVal,'String',num2str(sc));    
    scglob  = sc;
  
    norm    = get(HcNorm,'Value');
    if norm, 
      set(HcNorm,'String','yes','HorizontalAlignment','left');
      set(HcScale,'Enable','off');
    else 
      set(HcNorm,'String','no','HorizontalAlignment','left');
      set(HcScale,'Enable','on');    
    end;
      
    exttype = get(HcExt,'Value')-1;

    if exttype > 0           %Enable unless extension is periodic
      set(HcViewExt,'Enable','on');  
    else  
      set(HcViewExt,'Enable','off'); 
    end;    
    
    

%    %
%    % Compute wavelet spectrum
%    %
    if OK
      y = yorg;   %Restore original y XXXXXXXXXXXX
%      if exttype > 0
%    	%Call extend here XXXXXXXXXXXXXX
%        yfwt = fwt([y;flipud(y)],D,d);
%      else
%	yfwt = fwt(y,D,d);    
%      end;
%      len = length(yfwt);      

      %
      % Denoise according to threshold
      %
      tpow      = get(HcEps,'Value');
      threshold = 10^tpow;   

%      yfwt_hp = yfwt(len/2^d+1:end);  %Truncate only high pass part
%      yfwt_hp = trunc(yfwt_hp,threshold);
%      yfwt(len/2^d+1:end) = yfwt_hp;
%      y = ifwt(yfwt,D,d);
%    
%      if exttype > 0
%	y = y(1:end/2);   %Generalize this XXXXXXXXXXXXXXX
%      end;
    
      if tpow <= tpowmin
        set(HcEpsVal,'String',num2str(0));      
	threshold = 0;  
      else
        threshold = 10^tpow;    
	prec = 10^(-tpowmin); %Precision for presentation only
        set(HcEpsVal,'String',num2str(round(prec*threshold)/prec));    
      end;

      %
      % Plot original function
      %
      figure(H1);subplot(2,1,2);plot(x,y,'k-');%Plot data
      title(name);

      if ~isempty(xlab)
	xlabel(xlab)
      end;  

      %
      % Plot wavelet decomposition
      %
      if ~H2
        H2=figure;
      
        % Make initial H2 window taller and move it a bit to the right
        %

	H2dat = get(H2);
	pos2 = H2dat.Position;
	pos2(4) = pos2(4)*1.8;  %Increase height 
	pos2(2) = 10;           %Position figure close to bottom
	pos2(1) = pos2(1)*1.8;  %Move a bit to the right
	set(H2,'Position',pos2);
      else
        figure(H2);	
      end;
	
      %	 
      % Plot decomposition	
      %
      [yfwt,P,err,ratio,abs_thres] = plotw(x,y,D,d,exttype,norm,sc,threshold,name,xlab);    

%      if ~isempty(xlab)
%	xlabel(xlab)
%      end;  
%      if ~isempty(name)
%        H=get(gcf);
%      end;
      
      
      %Update wavelet spectrum
      if H3
%        figure(H3);ploth(yfwt,d);title(['Wavelet coefficients'])    
        figure(H3);plot(yfwt);title(['Wavelet coefficients'])    
      end;

      %Update view of extension
      if H4
        if exttype == 0
	  figure(H4); delete(H4); H4=0;
	else  
          %
          % Extend and call plotw with *periodic extension*
          %
          [xext,yext]=extend(x,y,D,d,exttype);
	  ename = [name ' + its extension'];
          figure(H4);plotw(xext,yext,D,d,0,norm,sc,threshold,ename,xlab);  
        end;
      end;
      
      %Output error and compression ratio
      prec = 100;
      set(HcErrVal,'String',num2str(err,'%5.1e'));    
      set(HcCompVal,'String',num2str(round(prec*ratio)/prec));          
      prec = 1000;
      set(HcThrVal,'String',num2str(round(prec*abs_thres)/prec));
       		
      figure(H1);  %Maybe superfluous xxxxxx      
    end;  
  
  
  %-------------------------------------------------------------
  % Compute wavelet coefficients and determine initial normalization
  %-------------------------------------------------------------
  %
  % HORRIBLE HACK. CHECK THIS LATER XXXXXXXXXXXXXXXXXXXXXXXXXXXX
  %
  elseif strcmp(sw,'fwt')
    D       = 2*get(HcD,'Value');    
    d       = get(HcDepth,'Value');
    exttype = get(HcExt,'Value')-1;          
      
    if OK
      d1 = d+1;  
      if exttype 
	yfwt = fwt([y;flipud(y)],D,d);  
	
	%Remove reflected part from wavelet spectrum  
	M = length(yfwt);      
	P = [];
	
	wsize = 0;
	last  = M;
	for j = 1:d1 
	  first = last/2;
	  if j < d1
	    Temp = yfwt(first+1:last);
	  else
	    Temp = yfwt(1:last);	    % DC-term     
	  end; 
	  T = Temp(1:end/2);        %Take only half to remove reflection.
	  if j < d1
	    wsize = max(wsize,max(abs((T)))); %Compute max size of detail levels
	  end;
	  P = [T;P];    % Aggregate new vector of wavelet coefs
	  
	  last = first;
	end;  		   
	yfwt = P;
      else
	yfwt = fwt(y,D,d);    
	M = length(yfwt);            
	wsize = 0;
	last  = M;
	for j = 1:d 
	  first = last/2;
	  if j < d
	    T = yfwt(first+1:last);	
	    wsize = max(wsize,max(abs((T)))); 
		   %Compute max size of detail levels	
	  end; 
	  
	  last = first;
	end;  		   
      end;  
    end;
    if isempty(normglob)
      delta = wsize/max(abs(yfwt));
      if delta > 0.01
        norm = 0;        %Don't normalize
      else  
        norm = 1;        %Normalize      
      end;      
      set(HcNorm,'Value',norm);          
    end;

    
  %----------------------------------------------------------
  % Display wavelet spectrum
  %----------------------------------------------------------
  elseif strcmp(sw,'view')
    if ~H3
      H3=figure;
    else
      figure(H3);
    end;    
    
    % Update H3 in 'Userdata'
    %  
    Figures(3) = H3;
    Globals{1} = Figures;        
    set(H1,'UserData',Globals);  
    figure(H1);  
    
    %
    % Then call wd
    %
    wavemenu('wd');   %Plots wavelet spectrum if H3 is open
  
%    D       = 2*get(HcD,'Value');    
%    d       = get(HcDepth,'Value');
%    yfwt=fwt(y,D,d); 
%    figure(H3);ploth(x,yfwt,d);title(['Wavelet coefficients'])
    

  %----------------------------------------------------------
  % Display actual extension 
  %----------------------------------------------------------
  elseif strcmp(sw,'extension')
    if ~H4
      H4=figure;
    else
      figure(H4);
    end;    
    
    % Update H4 in 'Userdata'
    %  
    Figures(4) = H4;
    Globals{1} = Figures;        
    set(H1,'UserData',Globals);  
    figure(H1);  
    
    %
    % Then call wd
    %
    wavemenu('wd');   %Plots extension if H4 is open
  

        
    %
    % Get necessary parameters
    %
%%%%    tpow      = get(HcEps,'Value');
%%%%    threshold = 10^tpow;   
%%%%    sc      = round(10*get(HcScale,'Value'))/10;
%%%%    set(HcScaleVal,'String',num2str(sc));    
    
    
        
    
  %----------------------------------------------------------
  % Denoise
  %----------------------------------------------------------
%  elseif strcmp(sw,'denoise')
%    D       = 2*get(HcD,'Value');    
%    d       = get(HcDepth,'Value');
%    exttype = get(HcExt,'Value') - 1;          
%  
%    tpow    = get(HcEps,'Value');
%    threshold = 10^tpow    
%    set(HcEpsVal,'String',num2str(threshold));    
%
%    y = yorg;   %Restore original y
%         
%    if OK
%      if exttype > 0
%      
%        %Call extend here XXXXXXXXXXXXXX
%       
%	yfwt = fwt([y;flipud(y)],D,d);
%	len = length(yfwt);
%	yfwt_hp = yfwt(len/2^d+1:end);  %Truncate only high pass part
%	yfwt_hp = trunc(yfwt_hp,threshold);
%	yfwt(len/2^d+1:end) = yfwt_hp;
%	y    = ifwt(yfwt,D,d);
%	y = y(1:end/2);   %Generalize this XXXXXXXXXXXXXXX
%      else
%	yfwt = fwt(y,D,d);    
%	len = length(yfwt);
%	yfwt_hp = yfwt(len/2^d+1:end);  %Truncate only high pass part
%	yfwt_hp = trunc(yfwt_hp,threshold);
%	yfwt(len/2^d+1:end) = yfwt_hp;
%	y    = ifwt(yfwt,D,d);
%      end;
%    
%      %
%      % Update y in 'Userdata'
%      %  
%      Globals = get(gcf,'UserData');
%      Data     = Globals{4};  
%      Data(:,2) = y;
%      Globals{4} = Data;        
%      set(H1,'UserData',Globals);    
%    end;
    
      
  %----------------------------------------------------------
  % Cleanup
  %----------------------------------------------------------
  elseif strcmp(sw,'cleanup')
    if H4, figure(H4); delete(H4); end; %If figure was removed inter-   
    if H3, figure(H3); delete(H3); end; %If figure was removed inter- 
    if H2, figure(H2); delete(H2); end; %actively, it must be recreated 
    if H1, figure(H1); delete(H1); end; %before it can be deleted.
    H1 = 0;
  end;

  
  if H1, 
    set(H1,'pointer','arrow');     
  

    Figures = [H1 H2 H3 H4];
    Handles  = ... 
      [HcWDA HcView HcNorm HcExt HcD HcDepth HcScale HcScaleVal HcEps HcEpsVal HcViewExt HcErrVal HcCompVal HcThrVal]; 
    Numvars  = [rows norm exttype D d OK];
    Textvars = {name, xlab};
    Data     = [x y yorg];
    Globals = {Figures, Handles, Numvars, Textvars, Data};   
    set(H1,'UserData',Globals);    
  end;

  %
  % Save global data for other instances of wavemenu
  %
  xglob       = x;    
%  yglob       = y;       
  yglob       = yorg;  %Probably best to save original fct       
  xlabglob    = xlab; 
  nameglob    = name; 
  dglob       = d;
  Dglob       = D;
  normglob    = norm;
  exttypeglob = exttype;
end;  

  
  
  
  
  
  
  
    
  %ypos0 = ypos0 - ywid0;			 			 
  %HcD = uicontrol(H1,'Style','slider',...
  %                   'Position',[xpos1 ypos0 2*xwid0 ywid0],...
  %		   'Min',1,'Max',25,'Value',D/2,...
  %                   'Callback','wavemenu(''wd'');');
  %                  'String','Set wavelet genus',...



